/*
 * ReversedBadCharacterShiftTable.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#include "ReversedBadCharacterShiftTable.h"

#include <tree/ranked/PrefixRankedBarPattern.h>
#include <tree/ranked/PrefixRankedBarNonlinearPattern.h>
#include <tree/ranked/PrefixRankedPattern.h>
#include <tree/ranked/PrefixRankedNonlinearPattern.h>

namespace arbology {

namespace exact {

std::map < alphabet::RankedSymbol, size_t > ReversedBadCharacterShiftTable::bcs ( const tree::RankedTreeWrapper & pattern ) {
	return dispatch ( pattern.getData ( ) );
}

std::map < alphabet::RankedSymbol, size_t > ReversedBadCharacterShiftTable::bcs ( const tree::PrefixRankedBarPattern & pattern ) {
	return bcs ( tree::PrefixRankedBarNonlinearPattern ( pattern ) );
}

auto ReversedBadCharacterShiftTablePrefixRankedBarPattern = ReversedBadCharacterShiftTable::RegistratorWrapper < std::map < alphabet::RankedSymbol, size_t >, tree::PrefixRankedBarPattern > ( ReversedBadCharacterShiftTable::bcs );

std::map < alphabet::RankedSymbol, size_t > ReversedBadCharacterShiftTable::bcs ( const tree::PrefixRankedBarNonlinearPattern & pattern ) {
	const std::set < alphabet::RankedSymbol > & alphabet = pattern.getAlphabet ( );

	std::map < alphabet::RankedSymbol, size_t > bcs;

	 // initialisation of bcs table to the size of the pattern
	for ( const alphabet::RankedSymbol & symbol : alphabet ) {
		if ( ( symbol == pattern.getSubtreeWildcard ( ) ) || ( pattern.getNonlinearVariables ( ).count ( symbol ) ) || ( symbol == pattern.getVariablesBar ( ) ) ) continue;

		bcs.insert ( std::make_pair ( symbol, pattern.getContent ( ).size ( ) ) );
	}

	 // find the distance between the beginning of the pattern and the index
	 // of the first symbol representing the variable's bar
	unsigned firstSBarOffset = pattern.getContent ( ).size ( ) + 1;

	for ( int i = ( int ) pattern.getContent ( ).size ( ) - 1; i >= 0; i-- )
		if ( pattern.getContent ( )[i] == pattern.getVariablesBar ( ) )
			firstSBarOffset = i;

	// limit the shift by occurrence of the last variable

	for ( const alphabet::RankedSymbol & symbol : alphabet ) {
		if ( ( symbol == pattern.getSubtreeWildcard ( ) ) || ( pattern.getNonlinearVariables ( ).count ( symbol ) ) || ( symbol == pattern.getVariablesBar ( ) ) ) continue;

		size_t tmp = firstSBarOffset;

		if ( pattern.getBars ( ).count ( symbol ) )
			 // size of the smallest subtree containing given terminal depend
			 // on the arity of the terminal
			tmp += symbol.getRank ( ).getData ( ) * 2;
		else if ( tmp >= 2 )
			 // bar symbols match the variable bar which is one symbol after
			 // the last variable, conditioned because of the case S S| where
			 // the -1 would cause shift by 0 -- illegal
			tmp -= 1;

		if ( bcs[symbol] > tmp )
			bcs[symbol] = tmp;
	}

	 // limit the shift by position of symbols within the pattern
	for ( unsigned i = pattern.getContent ( ).size ( ) - 1; i >= 1; i-- ) { // first symbol is not concerned
		if ( ( pattern.getContent ( )[i] == pattern.getSubtreeWildcard ( ) ) || ( pattern.getNonlinearVariables ( ).count ( pattern.getContent ( )[i] ) ) || ( pattern.getContent ( )[i] == pattern.getVariablesBar ( ) ) ) continue;

		size_t tmp = i;

		if ( bcs[pattern.getContent ( )[i]] > tmp )
			bcs[pattern.getContent ( )[i]] = tmp;
	}

	return bcs;
}

auto ReversedBadCharacterShiftTablePrefixRankedBarNonlinearPattern = ReversedBadCharacterShiftTable::RegistratorWrapper < std::map < alphabet::RankedSymbol, size_t >, tree::PrefixRankedBarNonlinearPattern > ( ReversedBadCharacterShiftTable::bcs );

std::map < alphabet::RankedSymbol, size_t > ReversedBadCharacterShiftTable::bcs ( const tree::PrefixRankedPattern & pattern ) {
	return bcs ( tree::PrefixRankedNonlinearPattern ( pattern ) );
}

auto ReversedBadCharacterShiftTablePrefixRankedPattern = ReversedBadCharacterShiftTable::RegistratorWrapper < std::map < alphabet::RankedSymbol, size_t >, tree::PrefixRankedPattern > ( ReversedBadCharacterShiftTable::bcs );

std::map < alphabet::RankedSymbol, size_t > ReversedBadCharacterShiftTable::bcs ( const tree::PrefixRankedNonlinearPattern & pattern ) {
	const std::set < alphabet::RankedSymbol > & alphabet = pattern.getAlphabet ( );

	std::map < alphabet::RankedSymbol, size_t > bcs;

	 // initialisation of bcs table to the size of the pattern
	for ( const alphabet::RankedSymbol & symbol : alphabet ) {
		if ( symbol == pattern.getSubtreeWildcard ( ) || pattern.getNonlinearVariables ( ).count ( symbol ) ) continue;

		bcs.insert ( std::make_pair ( symbol, pattern.getContent ( ).size ( ) ) );
	}

	 // find the distance between the beginning of the pattern and the index
	 // of the first symbol representing the variable's bar
	unsigned firstSOffset = pattern.getContent ( ).size ( ) + 1;

	for ( int i = ( int ) pattern.getContent ( ).size ( ) - 1; i >= 0; i-- )
		if ( pattern.getContent ( )[i] == pattern.getSubtreeWildcard ( ) || pattern.getNonlinearVariables ( ).count ( pattern.getContent ( )[i] ) )
			firstSOffset = i;

	if ( firstSOffset == 0 ) firstSOffset = 1;

	 // limit the shift by occurrence of the last variable
	for ( const alphabet::RankedSymbol & symbol : alphabet ) {
		if ( symbol == pattern.getSubtreeWildcard ( ) || pattern.getNonlinearVariables ( ).count ( symbol ) ) continue;

		if ( bcs[symbol] > firstSOffset )
			bcs[symbol] = firstSOffset;
	}

	 // limit the shift by position of symbols within the pattern
	for ( unsigned i = pattern.getContent ( ).size ( ) - 1; i >= 1; i-- ) { // first symbol is not concerned
		if ( pattern.getContent ( )[i] == pattern.getSubtreeWildcard ( ) || pattern.getNonlinearVariables ( ).count ( pattern.getContent ( )[i] ) ) continue;
		size_t tmp = i;

		if ( bcs[pattern.getContent ( )[i]] > tmp )
			bcs[pattern.getContent ( )[i]] = tmp;
	}

	return bcs;
}

auto ReversedBadCharacterShiftTablePrefixRankedNonlinearPattern = ReversedBadCharacterShiftTable::RegistratorWrapper < std::map < alphabet::RankedSymbol, size_t >, tree::PrefixRankedNonlinearPattern > ( ReversedBadCharacterShiftTable::bcs );

} /* namespace exact */

} /* namespace arbology */
