/*
 * GenerateUpToLength.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#include "GenerateUpToLength.h"

#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

#include <deque>

namespace grammar {

namespace generate {

template<class T>
std::set<string::LinearString> GenerateUpToLength::generate( const T & grammar, unsigned length ) {
	std::set<string::LinearString> res;

	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> rules = grammar.getRawRules();
	if(grammar.getGeneratesEpsilon()) {
		res.insert(string::LinearString {});
		rules[grammar.getInitialSymbol()].erase(std::vector<alphabet::Symbol> {});
	}

	std::deque<std::pair<std::vector<alphabet::Symbol>, std::vector<alphabet::Symbol>>> data;
	data.push_back(std::make_pair(std::vector<alphabet::Symbol>{}, std::vector<alphabet::Symbol>{grammar.getInitialSymbol()}));
	while(!data.empty()) {
		std::pair<std::vector<alphabet::Symbol>, std::vector<alphabet::Symbol>> item = std::move(data.back());
		data.pop_back();

		alphabet::Symbol nonterminal = std::move(item.second.back());
		item.second.pop_back();
		if(rules.find(nonterminal) == rules.end()) continue;

		for(const std::vector<alphabet::Symbol>& rhs : rules.find(nonterminal)->second) {
			if(item.first.size() + item.second.size() + rhs.size() > length) continue;

			std::vector<alphabet::Symbol> newStack(item.second);
			std::vector<alphabet::Symbol> newString(item.first);
			newStack.insert(newStack.end(), rhs.rbegin(), rhs.rend());

			while(!newStack.empty() && grammar.getTerminalAlphabet().count(newStack.back())) {
				newString.push_back(std::move(newStack.back()));
				newStack.pop_back();
			}

			if(newStack.empty()) {
				res.insert(string::LinearString(newString));
			} else {
				data.push_back(std::make_pair(std::move(newString), std::move(newStack)));
			}
		}
	}
	return res;
}

auto GenerateUpToLengthEpsilonFreeCFG = GenerateUpToLength::RegistratorWrapper<std::set<string::LinearString>, grammar::EpsilonFreeCFG>(GenerateUpToLength::generate);
auto GenerateUpToLengthGNF = GenerateUpToLength::RegistratorWrapper<std::set<string::LinearString>, grammar::GNF>(GenerateUpToLength::generate);
auto GenerateUpToLengthCNF = GenerateUpToLength::RegistratorWrapper<std::set<string::LinearString>, grammar::CNF>(GenerateUpToLength::generate);
auto GenerateUpToLengthLeftRG = GenerateUpToLength::RegistratorWrapper<std::set<string::LinearString>, grammar::LeftRG>(GenerateUpToLength::generate);
auto GenerateUpToLengthRightRG = GenerateUpToLength::RegistratorWrapper<std::set<string::LinearString>, grammar::RightRG>(GenerateUpToLength::generate);

std::set<string::LinearString> GenerateUpToLength::generate(const grammar::Grammar& grammar, unsigned length) {
	return dispatch(grammar.getData(), length);
}

} /* namespace generate */

} /* namespace grammar */

