/*
 * DeterministicLL1Grammar.cpp
 *
 *  Created on: 9. 6. 2015
 *	  Author: Jan Travnicek
 */

#include "DeterministicLL1Grammar.h"
#include "LL1ParseTable.h"
#include "HandleFirstFirstConflict.h"
#include "HandleFirstFollowConflict.h"

#include <grammar/ContextFree/CFG.h>
#include <exception/CommonException.h>
#include <algorithm>

#include "../properties/RecursiveNonterminal.h"

namespace grammar {

namespace parsing {

grammar::CFG DeterministicLL1Grammar::convert ( const grammar::CFG & param ) {

	if ( std::any_of ( param.getNonterminalAlphabet ( ).begin ( ), param.getNonterminalAlphabet ( ).end ( ), [&] ( const alphabet::Symbol & nonterminal ) {
				return grammar::properties::RecursiveNonterminal::isNonterminalRecursive ( param, nonterminal );
			} ) )
		throw exception::CommonException ( "Grammar contains left recursion" );

	grammar::CFG grammar = param;

	while ( true ) {
		std::map < std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > parseTable = LL1ParseTable::parseTable ( grammar );

		bool deterministic = true;

		for ( const std::pair < const std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > & elem : parseTable )
			if ( elem.second.size ( ) > 1 )
				if ( elem.first.first.is < string::Epsilon > ( ) )
					throw exception::CommonException ( "Cant handle conflict in epsilon" );

		for ( const std::pair < const std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > & elem : parseTable ) {
			if ( elem.first.first.is < string::Epsilon > ( ) ) continue;

			const alphabet::Symbol & terminal = elem.first.first.get < alphabet::Symbol > ( );
			const alphabet::Symbol & nonterminal = elem.first.second;
			const std::set < std::vector < alphabet::Symbol > > & rhsds = elem.second;

			if ( ( elem.second.size ( ) > 1 ) && std::all_of ( rhsds.begin ( ), rhsds.end ( ), [] ( const std::vector < alphabet::Symbol > & rhs ) {
						return rhs.size ( ) > 0;
					} ) ) {
				HandleFirstFirstConflict::handleFirstFirstConflict ( grammar, terminal, nonterminal, rhsds );
				deterministic = false;
				break;
			}
		}

		if ( !deterministic ) continue;

		for ( const std::pair < const std::pair < std::variant < alphabet::Symbol, string::Epsilon >, alphabet::Symbol >, std::set < std::vector < alphabet::Symbol > > > & elem : parseTable ) {
			if ( elem.first.first.is < string::Epsilon > ( ) ) continue;

			const alphabet::Symbol & terminal = elem.first.first.get < alphabet::Symbol > ( );
			const alphabet::Symbol & nonterminal = elem.first.second;
			const std::set < std::vector < alphabet::Symbol > > & rhsds = elem.second;

			if ( ( elem.second.size ( ) > 1 ) && std::any_of ( rhsds.begin ( ), rhsds.end ( ), [] ( const std::vector < alphabet::Symbol > & rhs ) {
						return rhs.size ( ) == 0;
					} ) ) {
				HandleFirstFollowConflict::handleFirstFollowConflict ( grammar, terminal, nonterminal, rhsds );
				deterministic = false;
				break;
			}
		}

		if ( !deterministic ) continue;

		return grammar;
	}
}

auto DeterministicLL1GrammarCFG = DeterministicLL1Grammar::RegistratorWrapper < grammar::CFG, grammar::CFG > ( DeterministicLL1Grammar::convert );

grammar::Grammar DeterministicLL1Grammar::convert ( const grammar::Grammar & grammar ) {
	return dispatch ( grammar.getData ( ) );
}

} /* namespace parsing */

} /* namespace grammar */
