/*
 * LRParser.cpp
 *
 *  Created on: 30. 4. 2016
 *	  Author: Martin Kocicka
 */

#include "LRParser.h"
#include <alphabet/EndSymbol.h>

#include <stack>

namespace grammar {

namespace parsing {

alphabet::Symbol LRParser::getEndOfInputSymbol ( grammar::CFG originalGrammar ) {
	return alphabet::createUniqueSymbol ( alphabet::Symbol ( alphabet::EndSymbol ( ) ), originalGrammar.getTerminalAlphabet ( ), originalGrammar.getNonterminalAlphabet ( ) );
}

grammar::CFG LRParser::getAugmentedGrammar ( grammar::CFG originalGrammar ) {
	alphabet::Symbol initialSymbol = alphabet::createUniqueSymbol ( originalGrammar.getInitialSymbol(), originalGrammar.getTerminalAlphabet ( ), originalGrammar.getNonterminalAlphabet ( ) );

	originalGrammar.addNonterminalSymbol ( initialSymbol );
	originalGrammar.addRule ( initialSymbol, { originalGrammar.getInitialSymbol ( ) } );
	originalGrammar.setInitialSymbol ( initialSymbol );

	return originalGrammar;
}

bool LRParser::parse ( LRActionTable actionTable, LRGotoTable gotoTable, automaton::State initialState, std::vector < alphabet::Symbol > input ) {
	std::stack < automaton::State > states;
	states.push ( initialState );

	unsigned currentPosition = 0;
	while ( true ) {
		LRActionTable::iterator actionIterator = actionTable.find ( { states.top ( ), input[currentPosition] } );
		if ( actionIterator == actionTable.end ( ) )
			return false;

		switch ( actionIterator->second.first ) {
			case LRAction::Shift:
				states.push ( actionIterator->second.second.get < automaton::State > ( ) );

				++currentPosition;
				break;

			case LRAction::Reduce: {
				std::pair < alphabet::Symbol, std::vector < alphabet::Symbol > > reduceBy = actionIterator->second.second.get < std::pair < alphabet::Symbol, std::vector < alphabet::Symbol > > > ( );
				for ( unsigned i = 0; i < reduceBy.second.size ( ); ++i ) {
					states.pop ( );
				}

				LRGotoTable::iterator nextStateIterator = gotoTable.find ( { states.top ( ), reduceBy.first } );
				if (nextStateIterator == gotoTable.end()) {
					return false;
				}

				states.push ( nextStateIterator->second );

				break;
			}

			case LRAction::Accept:
				return true;
				break;
		}
	}

	return false;
}

} /* namespace parsing */

} /* namespace grammar */
