/*
 * ToAutomatonGlushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "ToAutomatonGlushkov.h"

#include "label/Label.h"
#include "label/LabelPairLabel.h"
#include "label/ObjectLabel.h"
#include <automaton/Automaton.h>

#include "object/Object.h"

#include "../glushkov/GlushkovTraversal.h"

#include "../properties/RegExpEpsilon.h"
#include <exception/CommonException.h>

namespace regexp {

namespace convert {

automaton::NFA ToAutomatonGlushkov::convert(const regexp::RegExp& regexp) {
	return dispatch(regexp.getData());
}

automaton::NFA ToAutomatonGlushkov::convert(const regexp::UnboundedRegExp& regexp) {
	automaton::State q0( label::Label( label::LabelPairLabel( std::make_pair( label::labelFrom( 'q' ), label::labelFrom( 0 ) ) ) ) );
	automaton::NFA automaton( q0 );

	// step 1
	automaton.setInputAlphabet(regexp.getAlphabet());

	regexp::UnboundedRegExp indexedRegExp = regexp::GlushkovTraversal::indexate(regexp);
	// steps 2, 3, 4
	const std::set<regexp::UnboundedRegExpSymbol> first = regexp::GlushkovTraversal::first(indexedRegExp);
	const std::set<regexp::UnboundedRegExpSymbol> last = regexp::GlushkovTraversal::last(indexedRegExp);
	// \e in q0 check is in step 7

	// step 5
	for( auto const& symbol : indexedRegExp.getAlphabet())
		automaton.addState( automaton::State ( label::Label( label::ObjectLabel( alib::Object( symbol.getData() ) ) ) ) );

	// step 6
	for( auto const& symbol : first ) {
		automaton.addTransition( q0, regexp::GlushkovTraversal::getSymbolFromGlushkovPair(symbol.getSymbol()), automaton::State ( label::Label( label::ObjectLabel( alib::Object( symbol.getSymbol( ).getData() ) ) ) ) );
	}

	for(const auto& x : indexedRegExp.getAlphabet())
		for(const auto& f : regexp::GlushkovTraversal::follow(indexedRegExp, UnboundedRegExpSymbol ( x ))) {
			const alphabet::Symbol& p = x;
			const alphabet::Symbol& q = f.getSymbol();

			automaton.addTransition( automaton::State ( label::Label( label::ObjectLabel( alib::Object( p.getData( ) ) ) ) ), regexp::GlushkovTraversal::getSymbolFromGlushkovPair(q), automaton::State ( label::Label( label::ObjectLabel( alib::Object( q.getData( ) ) ) ) ) );
		}

	// step 7
	for( auto const& symbol : last ) {
		automaton.addFinalState( automaton::State ( label::Label( label::ObjectLabel( alib::Object( symbol.getSymbol( ).getData() ) ) ) ) );
	}

	if(regexp::properties::RegExpEpsilon::languageContainsEpsilon(regexp))
		automaton.addFinalState( q0 );

	return automaton;
}

auto ToAutomatonGlushkovUnboundedRegExp = ToAutomatonGlushkov::RegistratorWrapper<automaton::NFA, regexp::UnboundedRegExp>( ToAutomatonGlushkov::convert);

automaton::NFA ToAutomatonGlushkov::convert(const regexp::FormalRegExp& /* regexp */)
{
	throw exception::CommonException("Glushkov: Converting FormalRegExp NYI"); // TODO
}

auto ToAutomatonGlushkovFormalRegExp = ToAutomatonGlushkov::RegistratorWrapper<automaton::NFA, regexp::FormalRegExp>(ToAutomatonGlushkov::convert);

} /* namespace convert */

} /* namespace regexp */
