/*
 * GlushkovTraversal.h
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#ifndef RTE_GLUSHKOVTRAVERSAL_H_
#define RTE_GLUSHKOVTRAVERSAL_H_

#include <set>
#include <map>
#include <vector>
#include <algorithm>

#include <rte/formal/FormalRTE.h>
#include <rte/formal/FormalRTEElement.h>
#include <rte/formal/FormalRTEAlternation.h>
#include <rte/formal/FormalRTESubstitution.h>
#include <rte/formal/FormalRTEIteration.h>
#include <rte/formal/FormalRTESymbol.h>
#include <rte/formal/FormalRTEEmpty.h>

#include <exception/CommonException.h>

#include <alphabet/RankedSymbol.h>

#include "GlushkovSymbol.h"
#include "GlushkovPair.h"

namespace rte {

class GlushkovTraversal {
public:
	/**
	 * @param re rte to probe
	 * @return all rteSymbols whichcan start the word.
	 */
	static std::set < GlushkovSymbol > first ( rte::FormalRTE const & re );

	/**
	 * @param re rte to probe
	 * @return all rteSymbols that can terminate the word.
	 */
	static std::set < GlushkovSymbol > last ( rte::FormalRTE const & re );

	/**
	 * @param re rte to probe
	 * @param symbol GlushkovSymbol for which we need the follow()
	 * @return all symbols that can follow specific symbol in word
	 */
	static std::set < std::vector < GlushkovSymbol > > follow ( rte::FormalRTE const & re, GlushkovSymbol const & symbol );

	/**
	 * @param re rte to probe
	 * @return symbols of rte tree in order of they occurence in rte.
	 */
	static std::set < GlushkovSymbol > getSymbols ( rte::FormalRTE const & re );

private:
	/**
	 * @param symbol ptr to symbol
	 * @param symbolSet set of gl.symbols
	 * @return GlushkovSymbol equivalent for rteSymbol pointer
	 */
	static GlushkovSymbol const & findSymbol ( rte::FormalRTESymbol const * const symbol, const std::set < GlushkovSymbol > & symbolSet );

	/**
	 * @return bool true if symbol pointer is in this subtree
	 */
	static bool pos ( GlushkovSymbol const & symbol, rte::FormalRTE const * const & node );

	static void getSymbols ( rte::FormalRTEElement const * const & node, const std::set < alphabet::RankedSymbol > & alphabetF, std::set < GlushkovSymbol > & posSet, int & i );

	static std::set < rte::FormalRTESymbol const * > first ( rte::FormalRTEElement const * const & node );
	static std::set < rte::FormalRTESymbol const * > first ( rte::FormalRTEAlternation const * const & node );
	static std::set < rte::FormalRTESymbol const * > first ( rte::FormalRTESubstitution const * const & node );
	static std::set < rte::FormalRTESymbol const * > first ( rte::FormalRTEIteration const * const & node );
	static std::set < rte::FormalRTESymbol const * > first ( rte::FormalRTESymbol const * const & node );
	static std::set < rte::FormalRTESymbol const * > first ( rte::FormalRTEEmpty const * const & node );

	static bool pos ( rte::FormalRTEElement const * const & node, rte::FormalRTESymbol const * const & symbSearch );
	static bool pos ( rte::FormalRTEAlternation const * const & node, rte::FormalRTESymbol const * const & symbSearch );
	static bool pos ( rte::FormalRTESubstitution const * const & node, rte::FormalRTESymbol const * const & symbSearch );
	static bool pos ( rte::FormalRTEIteration const * const & node, rte::FormalRTESymbol const * const & symbSearch );
	static bool pos ( rte::FormalRTESymbol const * const & node, rte::FormalRTESymbol const * const & symbSearch );
	static bool pos ( rte::FormalRTEEmpty const * const & node, rte::FormalRTESymbol const * const & symbSearch );

	static std::set < std::vector < rte::FormalRTESymbol const * > > follow ( rte::FormalRTEElement const * const & node, rte::FormalRTESymbol const * const & symbptr, const std::set<alphabet::RankedSymbol>& alphabetK, std::map < alphabet::RankedSymbol, std::set < rte::FormalRTESymbol const * > > & subM );
	static std::set < std::vector < rte::FormalRTESymbol const * > > follow ( rte::FormalRTEAlternation const * const & node, rte::FormalRTESymbol const * const & symbptr, const std::set<alphabet::RankedSymbol>& alphabetK, std::map < alphabet::RankedSymbol, std::set < rte::FormalRTESymbol const * > > & subM );
	static std::set < std::vector < rte::FormalRTESymbol const * > > follow ( rte::FormalRTESubstitution const * const & node, rte::FormalRTESymbol const * const & symbptr, const std::set<alphabet::RankedSymbol>& alphabetK, std::map < alphabet::RankedSymbol, std::set < rte::FormalRTESymbol const * > > & subM );
	static std::set < std::vector < rte::FormalRTESymbol const * > > follow ( rte::FormalRTEIteration const * const & node, rte::FormalRTESymbol const * const & symbptr, const std::set<alphabet::RankedSymbol>& alphabetK, std::map < alphabet::RankedSymbol, std::set < rte::FormalRTESymbol const * > > & subM );
	static std::set < std::vector < rte::FormalRTESymbol const * > > follow ( rte::FormalRTESymbol const * const & node, rte::FormalRTESymbol const * const & symbptr, const std::set<alphabet::RankedSymbol>& alphabetK, std::map < alphabet::RankedSymbol, std::set < rte::FormalRTESymbol const * > > & subM );
	static std::set < std::vector < rte::FormalRTESymbol const * > > follow ( rte::FormalRTEEmpty const * const & node, rte::FormalRTESymbol const * const & symbptr, const std::set<alphabet::RankedSymbol>& alphabetK, std::map < alphabet::RankedSymbol, std::set < rte::FormalRTESymbol const * > > & subM );
};

} /* namespace rte */

#endif /* RTE_GLUSHKOVTRAVERSAL_H_ */
