/*
 * ToPostfixPushdownAutomatonGlushkov.h
 *
 *  Created on: 11. 4. 2016
 *	  Author: Tomas Pecka
 */

#include "ToPostfixPushdownAutomatonGlushkov.h"

#include "label/Label.h"
#include "label/LabelPairLabel.h"
#include "label/ObjectLabel.h"
#include "alphabet/BottomOfTheStackSymbol.h"
#include "alphabet/EndSymbol.h"

#include <automaton/Automaton.h>

#include "object/Object.h"
#include "global/GlobalData.h"

#include "../GlushkovTraversal.h"

namespace rte {

namespace convert {

automaton::NPDA ToPostfixPushdownAutomatonGlushkov::convert ( const rte::RTE & rte ) {
	return dispatch ( rte.getData ( ) );
}

std::vector < alphabet::Symbol > phi ( const std::vector < GlushkovSymbol > & follow, const std::map < GlushkovSymbol, alphabet::Symbol > & pos_stack_map ) {
	std::vector < alphabet::Symbol > ret;

	for ( const GlushkovSymbol & f : follow ) {
		auto it = pos_stack_map.find ( f );

		if ( it == pos_stack_map.end ( ) )
			throw exception::CommonException ( "GlushkovRTE: phi: symbol not in pos. Probably constant? Invalid RTE!" );

		ret.push_back ( it->second );
	}

	return ret;
}

automaton::NPDA ToPostfixPushdownAutomatonGlushkov::convert ( const rte::FormalRTE & rte ) {
	automaton::State q ( label::labelFrom ( 'q' ) );
	automaton::State f ( label::labelFrom ( 'f' ) );
	automaton::NPDA automaton ( q, alphabet::Symbol { alphabet::BottomOfTheStackSymbol::BOTTOM_OF_THE_STACK } );

	 // step 3
	automaton.addState ( f );
	automaton.addFinalState ( f );

	 // step 4
	for ( const alphabet::RankedSymbol & symbol : rte.getAlphabet ( ) )
		automaton.addInputSymbol ( symbol.getSymbol ( ) );

	automaton.addInputSymbol ( alphabet::Symbol { alphabet::EndSymbol::END } );

	std::set < GlushkovSymbol > posSet = rte::GlushkovTraversal::getSymbols ( rte );
	std::set < GlushkovSymbol > firstSet = rte::GlushkovTraversal::first ( rte );
	std::map < GlushkovSymbol, std::set < std::vector < GlushkovSymbol > > > followSet;

	for ( const auto & symbol : posSet )
		followSet.insert ( std::make_pair ( symbol, rte::GlushkovTraversal::follow ( rte, symbol ) ) );

	 /* DEBUG */
	if ( common::GlobalData::verbose ) {
		std::cerr << "Pos:" << std::endl;

		for ( const auto & symbol : posSet ) std::cerr << "\t" << symbol.getInputSymbol ( ) << " id=" << symbol.getId ( ) << " ptr=" << symbol.getSymbolPtr ( ) << std::endl;

		std::cerr << "First:" << std::endl;

		for ( const auto & symbol : firstSet ) std::cerr << "\t" << symbol.getInputSymbol ( ) << " id=" << symbol.getId ( ) << " ptr=" << symbol.getSymbolPtr ( ) << std::endl;

		std::cerr << "Follow:" << std::endl;

		for ( const auto & kv : followSet ) {
			std::cerr << "\t" << kv.first.getInputSymbol ( ) << " id=" << kv.first.getId ( ) << " ptr=0x" << kv.first.getSymbolPtr ( ) << std::endl;

			for ( const auto & f : kv.second ) {
				for ( const auto & symbol : f )
					std::cerr << "\t\t" << symbol.getInputSymbol ( ) << " id=" << symbol.getId ( ) << " ptr=" << symbol.getSymbolPtr ( ) << std::endl;

				std::cerr << std::endl;
			}
		}
	}

	/* DEBUG END */

	std::map < GlushkovSymbol, alphabet::Symbol > pos_stack_map;

	for ( const GlushkovSymbol & gsymb : posSet ) {
		auto label = label::Label ( label::LabelPairLabel ( std::make_pair ( label::Label ( label::ObjectLabel ( alib::Object ( gsymb.getInputSymbol ( ).getSymbol ( ).getData ( ) ) ) ), label::labelFrom ( gsymb.getId ( ) ) ) ) );

		pos_stack_map.insert ( std::make_pair ( gsymb, alphabet::Symbol ( alphabet::LabeledSymbol ( label ) ) ) );
		automaton.addPushdownStoreSymbol ( alphabet::Symbol ( alphabet::LabeledSymbol ( label ) ) );
	}

	for ( const GlushkovSymbol & symb : posSet ) {
		auto it = pos_stack_map.find ( symb );

		if ( it == pos_stack_map.end ( ) )
			throw exception::CommonException ( "GlushkovRTE: fail" );

		if ( symb.getInputSymbol ( ).getRank ( ) == primitive::Unsigned ( 0 ) )
			automaton.addTransition ( q, symb.getInputSymbol ( ).getSymbol ( ), { }, q, { it->second } );
		else
			for ( const std::vector < GlushkovSymbol > & f : followSet[symb] ) {
				std::vector < alphabet::Symbol > fstring = phi ( f, pos_stack_map );
				std::reverse ( fstring.begin ( ), fstring.end ( ) );
				automaton.addTransition ( q, symb.getInputSymbol ( ).getSymbol ( ), fstring, q, { it->second } );
			}

	}

	for ( const GlushkovSymbol & symb : firstSet ) {
		auto it = pos_stack_map.find ( symb );

		if ( it == pos_stack_map.end ( ) )
			throw exception::CommonException ( "GlushkovRTE: fail2" );

		automaton.addTransition ( q, alphabet::Symbol { alphabet::EndSymbol::END }, { it->second, alphabet::Symbol { alphabet::BottomOfTheStackSymbol::BOTTOM_OF_THE_STACK }
								  }, f, { } );
	}

	return automaton;
}

auto ToAutomatonGlushkovFormalRegExp = ToPostfixPushdownAutomatonGlushkov::RegistratorWrapper < automaton::NPDA, rte::FormalRTE > ( ToPostfixPushdownAutomatonGlushkov::convert );

/*
 * automaton::NPDA ToPostfixPushdownAutomatonGlushkov::convert(const rte::UnboundedRTE& rte)
 * {
 *  throw exception::CommonException("Glushkov: Converting Unbounded RTE to postfix rhPDA NYI"); // TODO
 * }
 *
 * auto ToAutomatonGlushkovUnboundedRegExp = ToAutomatonGlushkov::RegistratorWrapper<automaton::NPDA, rte::UnboundedRTE>(ToAutomatonGlushkov::getInstance(), ToAutomatonGlushkov::convert);
 */
} /* namespace convert */

} /* namespace rte */
