/*
 * RandomSubstringFactory.cpp
 *
 *  Created on: 27. 3. 2014
 *	  Author: Jan Travnicek
 */

#include "RandomSubstringFactory.h"

#include <algorithm>
#include <random>
#include <exception/CommonException.h>

#include <string/LinearString.h>

namespace string {

namespace generate {

string::String RandomSubstringFactory::generateSubstring ( size_t size, const string::String & v ) {
	return dispatch ( size, v.getData ( ) );
}

string::LinearString RandomSubstringFactory::generateSubstring ( size_t size, const string::LinearString & string ) {
	if ( size > string.getContent ( ).size ( ) )
		throw exception::CommonException ( "String not long enough" );

	size_t begin = std::random_devices::semirandom ( ) % ( string.getContent ( ).size ( ) - size + 1 );

	std::vector < alphabet::Symbol > data ( string.getContent ( ).begin ( ) + begin, string.getContent ( ).begin ( ) + begin + size );

	return LinearString {
			   string.getAlphabet ( ), data
	};
}

auto RandomSubstringFactoryLinearString = RandomSubstringFactory::RegistratorWrapper < string::LinearString, string::LinearString > ( RandomSubstringFactory::generateSubstring );

} /* namespace generate */

} /* namespace string */
