/*
 * NormalizeAlphabet.cpp
 *
 *  Created on: Dec 9, 2013
 *	Author: Jan Travnicek
 */

#include "NormalizeAlphabet.h"

#include <map>
#include <vector>
#include <set>

#include "alphabet/Symbol.h"

namespace string {

namespace simplify {

string::String NormalizeAlphabet::normalize(const string::String& string) {
	return dispatch(string.getData());
}

string::LinearString NormalizeAlphabet::normalize(const string::LinearString& string) {
	int counter = 0;
	std::map<alphabet::Symbol, char > normalizationData;

	for(const alphabet::Symbol& symbol : string.getContent()) {
		if(normalizationData.find(symbol) == normalizationData.end()) {
			normalizationData.insert(std::make_pair(symbol, counter++));
		}
	}

	std::set<alphabet::Symbol> alphabet;
	for(const auto& symbol : normalizationData) {
		alphabet.insert(alphabet::symbolFrom((char) (symbol.second + 'a')));
	}

	std::vector<alphabet::Symbol> data;
	for(const alphabet::Symbol& symbol : string.getContent()) {
		data.push_back(alphabet::symbolFrom((char) (normalizationData.find(symbol)->second + 'a')));
	}

	string::LinearString result(alphabet, data);
	return result;
}

auto NormalizeAlphabetLinearString = NormalizeAlphabet::RegistratorWrapper<string::LinearString, string::LinearString>(NormalizeAlphabet::normalize);

} /* namespace simplify */

} /* namespace string */
