/*
 * Author: Radovan Cerveny
 */

#include "BackwardOracleMatching.h"
#include "FactorOracleAutomaton.h"

#include <string/LinearString.h>
#include <alphabet/Symbol.h>

#include <algorithm>
#include <map>
#include <bitset>
#include <measure>

namespace stringology {

namespace exact {

std::set < unsigned > BackwardOracleMatching::match ( const string::String & subject, const string::String & pattern ) {
    return dispatch ( subject.getData ( ), pattern.getData ( ) );
}

std::set < unsigned > BackwardOracleMatching::match ( const string::LinearString & subject, const string::LinearString & pattern ) {
    std::set < unsigned > occ;

    measurements::start ( "Preprocess", measurements::Type::PREPROCESS );

    auto patternData = pattern.getContent ( );

    reverse ( patternData.begin ( ), patternData.end ( ) );

    const string::LinearString reversedPattern ( std::move ( patternData ) );

    automaton::DFA factorOracle = FactorOracleAutomaton::construct ( reversedPattern );

    measurements::end ( );

    measurements::start ( "Algorithm", measurements::Type::ALGORITHM );

    const automaton::State failState = automaton::State ( -1 );

    size_t posInSubject = 0;

    while ( posInSubject <= subject.getContent ( ).size ( ) - pattern.getContent ( ).size ( ) ) {

        automaton::State currentState = factorOracle.getInitialState ( );

        size_t posInPattern = reversedPattern.getContent ( ).size ( );

        while ( posInPattern > 0 && currentState != failState ) {
            auto transition = factorOracle.getTransitions ( ).find ( { currentState, subject.getContent ( ).at ( posInSubject + posInPattern - 1 ) } );

            if ( transition == factorOracle.getTransitions ( ).end ( ) )
                currentState = failState;
            else
                currentState = transition->second;

            posInPattern--;
        }

        if ( currentState != failState )
             // Yay, there is match!!!
            occ.insert ( posInSubject );

        posInSubject += posInPattern + 1;
    }

    measurements::end ( );

    return occ;
}

auto BackwardOracleMatchingLinearStringLinearString = BackwardOracleMatching::RegistratorWrapper < std::set < unsigned >, string::LinearString, string::LinearString > ( BackwardOracleMatching::match );

} /* namespace exact */

} /* namespace stringology */
