/*
 * BadCharacterShiftTable.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka, Jan Travnicek
 */

#include "BadCharacterShiftTable.h"

#include <string/LinearString.h>
#include <alphabet/Symbol.h>

namespace stringology {

namespace exact {

std::map<alphabet::Symbol, size_t> BadCharacterShiftTable::bcs(const string::String& pattern) {
	return dispatch(pattern.getData());
}

std::map<alphabet::Symbol, size_t> BadCharacterShiftTable::bcs(const string::LinearString& pattern) {
	const std::set<alphabet::Symbol>& alphabet = pattern.getAlphabet();
	std::map<alphabet::Symbol, size_t> bcs;

	/* Initialization of BCS to the length of the needle. */
	for(const auto& symbol : alphabet)
		bcs.insert(std::make_pair(symbol, pattern.getContent().size()));

	/* Filling out BCS, ignoring last character. */
	for(size_t i = 0; i < pattern.getContent().size() - 1; i++)
		bcs[pattern.getContent().at(i)] = pattern.getContent().size() - i - 1;

	/*
	for(const auto& kv: bcs)
		std::cout << std::string(kv.first) << " " << kv.second << std::endl;
	for(const auto& s: string.getContent())
		std::cout << std::string(s);std::cout << std::endl;
	*/

	return bcs;
}

auto BadCharacterShiftTableLinearString = BadCharacterShiftTable::RegistratorWrapper<std::map<alphabet::Symbol, size_t>, string::LinearString>(BadCharacterShiftTable::bcs);

} /* namespace exact */

} /* namespace stringology */
