/*
 * ExactMatchingAutomaton.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#include "ExactMatchingAutomaton.h"
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactMatchingAutomaton::construct(const string::String& pattern) {
	return dispatch(pattern.getData());
}

automaton::NFA ExactMatchingAutomaton::construct(const string::LinearString& pattern) {
	automaton::NFA res(automaton::State(0));
	res.setInputAlphabet(pattern.getAlphabet());
	for(const alphabet::Symbol& symbol : pattern.getAlphabet()) {
		res.addTransition(automaton::State(0), symbol, automaton::State(0));
	}
	int i = 1;
	for(const alphabet::Symbol& symbol : pattern.getContent()) {
		res.addState(automaton::State(i));
		res.addTransition(automaton::State(i-1), symbol, automaton::State(i));
		i++;
	}
	res.addFinalState(automaton::State(i-1));
	return res;
}

auto ExactMatchingAutomatonLinearString = ExactMatchingAutomaton::RegistratorWrapper<automaton::NFA, string::LinearString>(ExactMatchingAutomaton::construct);

} /* namespace exact */

} /* namespace stringology */
