#include "ExtractRightContext.h"

#include "grammar/ContextFree/CFG.h"
#include "grammar/parsing/ExtractRightContext.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ExtractRightContext, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ExtractRightContext );

void ExtractRightContext::setUp ( ) {
}

void ExtractRightContext::tearDown ( ) {
}

void ExtractRightContext::testExtractRightContext ( ) {
	alphabet::Symbol S = alphabet::symbolFrom ( "S" );
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );

	grammar::CFG grammar ( S );

	grammar.setTerminalAlphabet ( { a, b, c } );
	grammar.setNonterminalAlphabet ( { S, A, C } );
	grammar.setInitialSymbol ( S );

	grammar.addRule ( S, std::vector < alphabet::Symbol > { b, c, A, C } );
	grammar.addRule ( S, std::vector < alphabet::Symbol > { b, b } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, c, A, b } );
	grammar.addRule ( C, { a, b } );
	grammar.addRule ( C, { c, C } );

	grammar::CFG res = grammar;
	grammar::parsing::ExtractRightContext::extractRightContext ( res, a, { A } );

	grammar::CFG comp ( S );

	comp.setTerminalAlphabet ( { a, b, c } );
	comp.setNonterminalAlphabet ( { S, A, C } );
	comp.setInitialSymbol ( S );

	comp.addRule ( S, std::vector < alphabet::Symbol > { b, c, A, a, b } );
	comp.addRule ( S, std::vector < alphabet::Symbol > { b, c, A, c, C } );
	comp.addRule ( S, std::vector < alphabet::Symbol > { b, b } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, c, A, b } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { a, b } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { c, C } );

	// std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}

void ExtractRightContext::testExtractRightContext2 ( ) {
	alphabet::Symbol S = alphabet::symbolFrom ( "S" );
	alphabet::Symbol X = alphabet::symbolFrom ( "X" );
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );

	grammar::CFG grammar ( S );

	grammar.setTerminalAlphabet ( { a, b, c } );
	grammar.setNonterminalAlphabet ( { S, X, A, C } );
	grammar.setInitialSymbol ( S );

	grammar.addRule ( S, std::vector < alphabet::Symbol > { b, c, X, C } );
	grammar.addRule ( S, std::vector < alphabet::Symbol > { b, b } );
	grammar.addRule ( X, std::vector < alphabet::Symbol > { b, A } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, c, A, b } );
	grammar.addRule ( C, { A, a, b } );
	grammar.addRule ( C, { c, C } );

	grammar::CFG res = grammar;
	grammar::parsing::ExtractRightContext::extractRightContext ( res, a, { A, X } );

	grammar::CFG comp ( S );

	comp.setTerminalAlphabet ( { a, b, c } );
	comp.setNonterminalAlphabet ( { S, X, A, C } );
	comp.setInitialSymbol ( S );

	comp.addRule ( S, std::vector < alphabet::Symbol > { b, c, X, A, a, b } );
	comp.addRule ( S, std::vector < alphabet::Symbol > { b, c, X, c, C } );
	comp.addRule ( S, std::vector < alphabet::Symbol > { b, b } );
	comp.addRule ( X, { b, A } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, c, A, b } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { A, a, b } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { c, C } );

	// std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}
