#include "HandleFirstFirstConflict.h"

#include "grammar/ContextFree/CFG.h"
#include "grammar/parsing/HandleFirstFirstConflict.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( HandleFirstFirstConflict, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( HandleFirstFirstConflict );

void HandleFirstFirstConflict::setUp ( ) {
}

void HandleFirstFirstConflict::tearDown ( ) {
}

void HandleFirstFirstConflict::testHandleFirstFirstConflict ( ) {
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol Ap = alphabet::symbolFrom ( "A'" );
	alphabet::Symbol B = alphabet::symbolFrom ( "B" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );

	grammar::CFG grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { C, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { d } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

	grammar::CFG res = grammar;
	grammar::parsing::HandleFirstFirstConflict::handleFirstFirstConflict ( res, a, A, { {a, B }, {C ,B} } );

	grammar::CFG comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, C, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { b, B, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { d } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );

	grammar::parsing::HandleFirstFirstConflict::handleFirstFirstConflict ( res, a, A, { {a, B }, {a, C, B} } );

	grammar::CFG comp2 ( A );

	comp2.setTerminalAlphabet ( { a, b, c, d } );
	comp2.setNonterminalAlphabet ( { A, Ap, B, C } );
	comp2.setInitialSymbol ( A );

	comp2.addRule ( A, std::vector < alphabet::Symbol > { a, Ap } );
	comp2.addRule ( A, std::vector < alphabet::Symbol > { b, B, B } );
	comp2.addRule ( Ap, std::vector < alphabet::Symbol > { B } );
	comp2.addRule ( Ap, std::vector < alphabet::Symbol > { C, B } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { d } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

//	std::cout << res << std::endl << comp2 << std::endl;

	CPPUNIT_ASSERT ( res == comp2 );
}

void HandleFirstFirstConflict::testHandleFirstFirstConflict2 ( ) {
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol Ap = alphabet::symbolFrom ( "A'" );
	alphabet::Symbol B = alphabet::symbolFrom ( "B" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );

	grammar::CFG grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { C, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { a } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { B, b } );

	grammar::CFG res = grammar;
	grammar::parsing::HandleFirstFirstConflict::handleFirstFirstConflict ( res, a, A, { {a, B}, {C, B}});
	grammar::parsing::HandleFirstFirstConflict::handleFirstFirstConflict ( res, a, A, { {a, B}, {a, C, B}, {B, b, B}});

	grammar::CFG comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < alphabet::Symbol > { a, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, C, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, b, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { c, B, b, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { a } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { B, b } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );

	grammar::parsing::HandleFirstFirstConflict::handleFirstFirstConflict ( res, a, A, { {a, B}, {a, C, B}, {a, b, B}});

	grammar::CFG comp2 ( A );

	comp2.setTerminalAlphabet ( { a, b, c, d } );
	comp2.setNonterminalAlphabet ( { A, Ap, B, C } );
	comp2.setInitialSymbol ( A );

	comp2.addRule ( A, std::vector < alphabet::Symbol > { a, Ap } );
	comp2.addRule ( A, std::vector < alphabet::Symbol > { c, B, b, B } );
	comp2.addRule ( Ap, std::vector < alphabet::Symbol > { B } );
	comp2.addRule ( Ap, std::vector < alphabet::Symbol > { C, B } );
	comp2.addRule ( Ap, std::vector < alphabet::Symbol > { b, B } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { c, B } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { a } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { B, b } );

//	std::cout << res << std::endl << comp2 << std::endl;

	CPPUNIT_ASSERT ( res == comp2 );
}
