#include "TopologicalSortTest.h"

#include "graph/sort/TopologicalSort.h"

#define CPPUNIT_ASSERT_EQUAL_INT(a, b) CPPUNIT_ASSERT_EQUAL(a, (int)b)

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GraphTopologicalSortTest, "graph" );
CPPUNIT_TEST_SUITE_REGISTRATION( GraphTopologicalSortTest );

void GraphTopologicalSortTest::testSort1()
{
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");
	graph::Node n6("n6");

	graph::AdjacencyListDirectedGraph dg;
	dg.addNode(n1);
	dg.addNode(n2);
	dg.addNode(n3);
	dg.addNode(n4);
	dg.addNode(n5);
	dg.addNode(n6);
	dg.addEdge(graph::DirectedEdge(n1, n2));
	dg.addEdge(graph::DirectedEdge(n1, n2, "multi-edge"));
	dg.addEdge(graph::DirectedEdge(n2, n3));
	dg.addEdge(graph::DirectedEdge(n3, n4));
	dg.addEdge(graph::DirectedEdge(n3, n6));
	dg.addEdge(graph::DirectedEdge(n4, n5));

	graph::sort::TopologicalSort::Result res;
	res = graph::sort::TopologicalSort::topologicalsort(dg);

	CPPUNIT_ASSERT_EQUAL_INT(6, res.size());

	CPPUNIT_ASSERT_EQUAL(n1, *res.begin());
	res.erase(res.begin());

	CPPUNIT_ASSERT_EQUAL(n2, *res.begin());
	res.erase(res.begin());

	CPPUNIT_ASSERT_EQUAL(n3, *res.begin());
	res.erase(res.begin());

	bool wasN4 = false;

	while (!res.empty()) {
		auto i = res.begin();
		if (*i == n4) {
			wasN4 = true;
		}
		if (*i == n5 && !wasN4) {
			CPPUNIT_FAIL("n4 must be before n5!");
		}
		res.erase(i);
	}
}

