/*
 * EndSymbol.cpp
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#include "EndSymbol.h"
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <XmlApi.hpp>
#include "UniqueSymbol.h"

namespace alphabet {

EndSymbol::EndSymbol() {

}

SymbolBase* EndSymbol::clone() const {
	return new EndSymbol(*this);
}

SymbolBase* EndSymbol::plunder() && {
	return new EndSymbol(std::move(*this));
}

int EndSymbol::compare(const EndSymbol&) const {
	return 0;
}

void EndSymbol::operator>>(std::ostream& out) const {
	out << "(EndSymbol)";
}

EndSymbol::operator std::string () const {
	return "$";
}

EndSymbol EndSymbol::END = EndSymbol ( );
Symbol EndSymbol::END_SYMBOL = Symbol ( EndSymbol ( ) );

EndSymbol EndSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, EndSymbol::getXmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, EndSymbol::getXmlTagName());
	return EndSymbol();
}

void EndSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(EndSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(EndSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* EndSymbol::next() const {
	return new UniqueSymbol(Symbol(*this), primitive::Integer(0));
}

SymbolBase* EndSymbol::inc() && {
	return new UniqueSymbol(Symbol(std::move(*this)), primitive::Integer(0));
}

} /* namespace alphabet */

namespace alib {

auto endSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::EndSymbol>();
auto endSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::EndSymbol>();

} /* namespace alib */
