/*
 * AutomatonToXMLComposer.cpp
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#include "AutomatonToXMLComposer.h"

#include <XmlApi.hpp>
#include "State.h"
#include "../../alphabet/RankedSymbol.h"
#include "Shift.h"
#include "../../string/StringClasses.h"
#include "../../regexp/RegExp.h"

namespace automaton {

void AutomatonToXMLComposer::composeStates(std::deque<sax::Token>& out, const std::set<State>& states) {
	out.emplace_back("states", sax::Token::TokenType::START_ELEMENT);
	for (const auto& state : states) {
		alib::xmlApi<State>::compose(out, state);
	}
	out.emplace_back("states", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeInputAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("inputAlphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("inputAlphabet", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeCallInputAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("callInputAlphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("callInputAlphabet", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeReturnInputAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("returnInputAlphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("returnInputAlphabet", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeLocalInputAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("localInputAlphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("localInputAlphabet", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeRankedInputAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::RankedSymbol>& symbols) {
	out.emplace_back(sax::Token("rankedInputAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::RankedSymbol>::compose(out, symbol);
	}
	out.emplace_back(sax::Token("rankedInputAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeInitialStates(std::deque<sax::Token>& out, const std::set<State>& states) {
	out.emplace_back("initialStates", sax::Token::TokenType::START_ELEMENT);
	for (const auto& state : states) {
		alib::xmlApi<label::Label>::compose(out, state.getName());
	}
	out.emplace_back("initialStates", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeInitialState(std::deque<sax::Token>& out, const State& state) {
	out.emplace_back("initialState", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<label::Label>::compose(out, state.getName());
	out.emplace_back("initialState", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeFinalStates(std::deque<sax::Token>& out, const std::set<State>& states) {
	out.emplace_back("finalStates", sax::Token::TokenType::START_ELEMENT);
	for (const auto& state : states) {
		alib::xmlApi<label::Label>::compose(out, state.getName());
	}
	out.emplace_back("finalStates", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composePushdownStoreAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("pushdownStoreAlphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("pushdownStoreAlphabet", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeInitialPushdownStoreSymbols(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("initialPushdownStoreSymbols", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("initialPushdownStoreSymbols", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeInitialPushdownStoreSymbol(std::deque<sax::Token>& out, const alphabet::Symbol& symbol) {
	out.emplace_back("initialPushdownStoreSymbol", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.emplace_back("initialPushdownStoreSymbol", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeOutputAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("outputAlphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("outputAlphabet", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTapeAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) {
	out.emplace_back("tapeAlphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("tapeAlphabet", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeBlankSymbol(std::deque<sax::Token>& out, const alphabet::Symbol& symbol) {
	out.emplace_back("blankSymbol", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.emplace_back("blankSymbol", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeBottomOfTheStackSymbol(std::deque<sax::Token>& out, const alphabet::Symbol& symbol) {
	out.emplace_back("bottomOfTheStackSymbol", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.emplace_back("bottomOfTheStackSymbol", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeInputToPushdownStoreOperation(std::deque<sax::Token>& out, const std::map<alphabet::Symbol, std::pair<std::vector<alphabet::Symbol>, std::vector<alphabet::Symbol> > >& operations) {
	out.emplace_back("inputToPushdownStoreOperations", sax::Token::TokenType::START_ELEMENT);
	for(const auto& pushdownStoreOperation : operations) {
		out.emplace_back("operation", sax::Token::TokenType::START_ELEMENT);

		alib::xmlApi<alphabet::Symbol>::compose(out, pushdownStoreOperation.first);

		composeTransitionPop(out, pushdownStoreOperation.second.first);
		composeTransitionPush(out, pushdownStoreOperation.second.second);

		out.emplace_back("operation", sax::Token::TokenType::END_ELEMENT);
	}
	out.emplace_back("inputToPushdownStoreOperations", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionTo(std::deque<sax::Token>& out, const State& state) {
	out.emplace_back("to", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<label::Label>::compose(out, state.getName());
	out.emplace_back("to", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionFrom(std::deque<sax::Token>& out, const State& state) {
	out.emplace_back("from", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<label::Label>::compose(out, state.getName());
	out.emplace_back("from", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionFrom(std::deque<sax::Token>& out, const std::vector<State> & states) {
	out.emplace_back(sax::Token("from", sax::Token::TokenType::START_ELEMENT));
	for (const auto& state : states) {
		alib::xmlApi<label::Label>::compose(out, state.getName());
	}
	out.emplace_back(sax::Token("from", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionShift(std::deque<sax::Token>& out, Shift shift) {
	out.emplace_back("shift", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(SHIFT_NAMES [shiftToInt(shift)], sax::Token::TokenType::CHARACTER);
	out.emplace_back("shift", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionPop(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) {
	out.emplace_back("pop", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("pop", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionSinglePop(std::deque<sax::Token>& out, const alphabet::Symbol& symbol) {
	out.emplace_back("pop", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.emplace_back("pop", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionPush(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) {
	out.emplace_back("push", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("push", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionSinglePush(std::deque<sax::Token>& out, const alphabet::Symbol& symbol) {
	out.emplace_back("push", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.emplace_back("push", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionInputSymbol(std::deque<sax::Token>& out, const alphabet::Symbol& symbol) {
	out.emplace_back("input", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.emplace_back("input", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionInputSymbol(std::deque<sax::Token>& out, const alphabet::RankedSymbol& symbol) {
	out.emplace_back(sax::Token("input", sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<alphabet::RankedSymbol>::compose(out, symbol);
	out.emplace_back(sax::Token("input", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionOutputSymbol(std::deque<sax::Token>& out, const alphabet::Symbol& symbol) {
	out.emplace_back("output", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	out.emplace_back("output", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionOutputSymbolMultiple(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) {
	out.emplace_back("output", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : symbols) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("output", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionInputEpsilonSymbol(std::deque<sax::Token>& out, const std::variant<string::Epsilon, alphabet::Symbol>& symbol) {
	out.emplace_back("input", sax::Token::TokenType::START_ELEMENT);
	if(symbol.is<string::Epsilon>()) {
		out.emplace_back("epsilon", sax::Token::TokenType::START_ELEMENT);
		out.emplace_back("epsilon", sax::Token::TokenType::END_ELEMENT);
	} else {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol.get<alphabet::Symbol>());
	}
	out.emplace_back("input", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionOutputEpsilonSymbol(std::deque<sax::Token>& out, const std::variant<string::Epsilon, alphabet::Symbol>& symbol) {
	out.emplace_back("output", sax::Token::TokenType::START_ELEMENT);
	if(symbol.is<string::Epsilon>()) {
		out.emplace_back("epsilon", sax::Token::TokenType::START_ELEMENT);
		out.emplace_back("epsilon", sax::Token::TokenType::END_ELEMENT);
	} else {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol.get<alphabet::Symbol>());
	}
	out.emplace_back("output", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionInputString(std::deque<sax::Token>& out, const string::LinearString& string) {
	out.emplace_back("input", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<string::LinearString>::compose(out, string);
	out.emplace_back("input", sax::Token::TokenType::END_ELEMENT);
}

void AutomatonToXMLComposer::composeTransitionInputRegexp(std::deque<sax::Token>& out, const regexp::RegExp& regexp) {
	out.emplace_back("input", sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<regexp::RegExp>::compose(out, regexp);
	out.emplace_back("input", sax::Token::TokenType::END_ELEMENT);
}

} /* namespace automaton */

