/*
 * State.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#ifndef STATE_H_
#define STATE_H_

#include <string>
#include <ostream>

#include "../../label/Label.h"
#include <XmlApi.hpp>

namespace automaton {

/**
 * Class representing state in an automaton.
 */
class State {
private:
	label::Label name;
public:
	explicit State(label::Label name);
	explicit State(int number);
	explicit State(int number1, int number2);
	explicit State(char character);
	explicit State(std::string name);
	explicit State(const char* name);

	const label::Label& getName() const;
	label::Label& getName();

	bool operator < (const State& other) const;
	bool operator == (const State& other) const;
	bool operator != (const State& other) const;
	int compare(const State& other) const;

	friend std::ostream& operator<<(std::ostream&, const State&);

	explicit operator std::string () const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "State";

		return xmlTagName;
	}

	static State parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;
};

} /* namespace automaton */

namespace std {

template<>
struct compare<automaton::State> {
	int operator()(const automaton::State& first, const automaton::State& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

namespace alib {

template<>
struct xmlApi<automaton::State> {
	static automaton::State parse(std::deque<sax::Token>::iterator& input);
	static bool first(const std::deque<sax::Token>::const_iterator& input);
	static void compose(std::deque<sax::Token>& output, const automaton::State& data);
};

} /* namespace alib */

#endif /* STATE_H_ */
