/*
 * GrammarToXMLComposer.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_TO_XML_COMPOSER_H_
#define GRAMMAR_TO_XML_COMPOSER_H_

#include <deque>
#include <vector>
#include <tuple>
#include <set>
#include <variant>
#include "../../alphabet/SymbolFeatures.h"
#include <sax/Token.h>

namespace grammar {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class GrammarToXMLComposer {
public:
	static void composeNonterminalAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols);
	static void composeTerminalAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols);
	static void composeInitialSymbol(std::deque<sax::Token>& out, const alphabet::Symbol& symbol);
	static void composeGeneratesEpsilon(std::deque<sax::Token>& out, bool generatesEpsilon);

	static void composeRuleLContext(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols);
	static void composeRuleLHS(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols);
	static void composeRuleRContext(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols);
	static void composeRuleSingleSymbolLHS(std::deque<sax::Token>& out, const alphabet::Symbol& symbol);
	static void composeRuleRHS(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols);
	static void composeRuleOneOrTwoSymbolsRHS(std::deque<sax::Token>& out, const std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>>& symbols);
	static void composeRuleGNFRHS(std::deque<sax::Token>& out, const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>& symbols);
	static void composeRuleLeftLGRHS(std::deque<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols);
	static void composeRuleRightLGRHS(std::deque<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>>& symbols);
	static void composeRuleLGRHS(std::deque<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols);
};

} /* namespace grammar */

#endif /* GRAMMAR_TO_XML_COMPOSER_H_ */

