/*
 * DirectedEdge.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "DirectedEdge.h"
#include "../../label/ObjectLabel.h"
#include <object/Void.h>

#include <sstream>

namespace graph {

DirectedEdge::DirectedEdge(Node from, Node to)
	: DirectedEdge(std::move(from), std::move(to), label::labelFrom('d'))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, label::Label name)
	: from(std::move(from))
	, to(std::move(to))
	, name(std::move(name))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, int number)
	: DirectedEdge(std::move(from), std::move(to), label::labelFrom(number))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, char character)
	: DirectedEdge(std::move(from), std::move(to), label::labelFrom(character))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, std::string name)
	: DirectedEdge(std::move(from), std::move(to), label::labelFrom(name))
{
}

DirectedEdge * DirectedEdge::clone() const
{
	return new DirectedEdge(*this);
}

DirectedEdge * DirectedEdge::plunder() &&
{
	return new DirectedEdge(std::move(*this));
}

const Node & DirectedEdge::getFromNode() const
{
	return from;
}

const Node & DirectedEdge::getToNode() const
{
	return to;
}

const label::Label & DirectedEdge::getName() const
{
	return name;
}

bool DirectedEdge::operator < (const DirectedEdge & other) const {
	return this->compare(other) < 0;
}

bool DirectedEdge::operator > (const DirectedEdge & other) const {
	return this->compare(other) > 0;
}

bool DirectedEdge::operator == (const DirectedEdge & other) const {
	return this->compare(other) == 0;
}

bool DirectedEdge::operator != (const DirectedEdge & other) const {
	return !(*this == other);
}

int DirectedEdge::compare(const DirectedEdge & other) const
{
	int res = name.getData().compare(other.name.getData());
	if (res != 0) return res;
	res = from.compare(other.from);
	if (res != 0) return res;
	return to.compare(other.to);
}

DirectedEdge::operator std::string() const
{
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

std::ostream & operator<<(std::ostream & out, const DirectedEdge & node)
{
	node.dump(out);
	return out;
}

void DirectedEdge::dump(std::ostream & os) const
{
	os << "(DirectedEdge " << from << " -> ( " << name << " ) " << to << ")";
}

} // namespace graph

