/*
 * SuffixTrieNodeTerminatingSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Stepan Plachy
 */

#ifndef SUFFIX_TRIE_NODE_TERMINATING_SYMBOL_H_
#define SUFFIX_TRIE_NODE_TERMINATING_SYMBOL_H_

#include "../../alphabet/Symbol.h"
#include "../../primitive/Unsigned.h"
#include <map>
#include <set>

namespace indexes {

class SuffixTrieTerminatingSymbol;

/**
 * Represents a node in the ranked tree. Contains name of the symbol.
 */
class SuffixTrieNodeTerminatingSymbol {
protected:
	std::map < alphabet::Symbol, SuffixTrieNodeTerminatingSymbol * > children;

	SuffixTrieNodeTerminatingSymbol * parent;

	/**
	 * Parent tree contanining this instance of RankedTree
	 */
	const SuffixTrieTerminatingSymbol * parentTree;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::attachTree()
	 */
	bool attachTree ( const SuffixTrieTerminatingSymbol * tree );

public:
	explicit SuffixTrieNodeTerminatingSymbol ( std::map < alphabet::Symbol, SuffixTrieNodeTerminatingSymbol * > children );

	SuffixTrieNodeTerminatingSymbol ( const SuffixTrieNodeTerminatingSymbol & other );
	SuffixTrieNodeTerminatingSymbol ( SuffixTrieNodeTerminatingSymbol && other ) noexcept;
	SuffixTrieNodeTerminatingSymbol & operator =( const SuffixTrieNodeTerminatingSymbol & other );
	SuffixTrieNodeTerminatingSymbol & operator =( SuffixTrieNodeTerminatingSymbol && other ) noexcept;
	~SuffixTrieNodeTerminatingSymbol ( ) noexcept;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::clone() const
	 */
	SuffixTrieNodeTerminatingSymbol * clone ( ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::plunder() const
	 */
	SuffixTrieNodeTerminatingSymbol * plunder ( ) &&;

	/**
	 * @copydoc RankedNode::computeMinimalAlphabet()
	 */
	std::set < alphabet::Symbol > computeMinimalAlphabet ( ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @return children
	 */
	const std::map < const alphabet::Symbol, const SuffixTrieNodeTerminatingSymbol * > & getChildren ( ) const;

	/**
	 * @return children
	 */
	const std::map < alphabet::Symbol, SuffixTrieNodeTerminatingSymbol * > & getChildren ( );

	SuffixTrieNodeTerminatingSymbol & getChild ( const alphabet::Symbol & symbol );

	const SuffixTrieNodeTerminatingSymbol & getChild ( const alphabet::Symbol & symbol ) const;

	bool hasChild ( const alphabet::Symbol & symbol ) const;

	SuffixTrieNodeTerminatingSymbol & addChild ( alphabet::Symbol symbol, SuffixTrieNodeTerminatingSymbol node );

	SuffixTrieNodeTerminatingSymbol * getParent ( );

	const SuffixTrieNodeTerminatingSymbol * getParent ( ) const;

	void swap ( SuffixTrieNodeTerminatingSymbol & other );

	int compare ( const SuffixTrieNodeTerminatingSymbol & ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::testSymbol() const
	 */
	bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::operator>>() const
	 */
	void operator >>( std::ostream & out ) const;

	friend std::ostream & operator <<( std::ostream &, const SuffixTrieNodeTerminatingSymbol & node );

	explicit operator std::string ( ) const;

	friend class SuffixTrieTerminatingSymbol;
};

} /* namespace indexes */

namespace std {

template < >
struct compare < indexes::SuffixTrieNodeTerminatingSymbol > {
	int operator ()( const indexes::SuffixTrieNodeTerminatingSymbol & first, const indexes::SuffixTrieNodeTerminatingSymbol & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* SUFFIX_TRIE_NODE_TERMINATING_SYMBOL_H_ */
