/*
 * String.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "String.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

namespace primitive {

String::String(std::string data) : data(std::move(data)) {

}

String::String(const char* data) : String((std::string) data) {

}

PrimitiveBase* String::clone() const {
	return new String(*this);
}

PrimitiveBase* String::plunder() && {
	return new String(std::move(*this));
}

const std::string& String::getData() const {
	return data;
}

std::string& String::getData() {
	return data;
}

int String::compare(const String& other) const {
	return data.compare(other.data);
}

void String::operator>>(std::ostream& out) const {
	out << "(String " << data << ")";
}

String::operator std::string() const {
	return data;
}

String String::parse(std::deque<sax::Token>::iterator& input) {
	return String(parseRaw(input));
}

std::string String::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, primitive::String::getXmlTagName());
	std::string data = "";
	if( sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::CHARACTER))
		data = sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, primitive::String::getXmlTagName());
	return data;
}


void String::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void String::compose(std::deque<sax::Token>& out, std::string primitive) {
	out.emplace_back(primitive::String::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(primitive, sax::Token::TokenType::CHARACTER);
	out.emplace_back(primitive::String::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

String String::next() const {
	return String(data + '\'');
}

void String::inc() {
	data.push_back('\'');
}

} /* namespace primitive */

namespace alib {

std::string xmlApi<std::string>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::String::parseRaw(input);
}

bool xmlApi<std::string>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, primitive::String::getXmlTagName());
}

void xmlApi<std::string>::compose(std::deque<sax::Token>& output, std::string data) {
	primitive::String::compose(output, data);
}

auto stringParserRegister = xmlApi<primitive::Primitive>::ParserRegister<primitive::String>();
auto stringParserRegister2 = xmlApi<alib::Object>::ParserRegister<primitive::String>();

} /* namespace alib */

