/*
 * RegExpFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpFromXMLParser.h"
#include <sax/FromXMLParserHelper.h>
#include <sax/ParserException.h>

#include "../RegExp.h"
#include "../RegExpClasses.h"

#include "../unbounded/UnboundedRegExpElements.h"
#include "../formal/FormalRegExpElements.h"
#include <XmlApi.hpp>

namespace regexp {

std::set < alphabet::Symbol > RegExpFromXMLParser::parseAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > alphabet;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabet.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );
	return alphabet;
}

std::rvalue_ref < UnboundedRegExpElement > RegExpFromXMLParser::parseUnboundedRegExpElement ( std::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseUnboundedRegExpEmpty ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" ) )
		return parseUnboundedRegExpEpsilon ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseUnboundedRegExpIteration ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseUnboundedRegExpAlternation ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" ) )
		return parseUnboundedRegExpConcatenation ( input );
	else
		return std::rvalue_ref < UnboundedRegExpElement > ( new UnboundedRegExpSymbol ( alib::xmlApi < alphabet::Symbol >::parse ( input ) ) );
}

std::rvalue_ref < UnboundedRegExpElement > RegExpFromXMLParser::parseUnboundedRegExpAlternation ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	UnboundedRegExpAlternation * alternation = new UnboundedRegExpAlternation ( );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alternation->appendElement ( parseUnboundedRegExpElement ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );
	return std::rvalue_ref < UnboundedRegExpElement > ( alternation );
}

std::rvalue_ref < UnboundedRegExpElement > RegExpFromXMLParser::parseUnboundedRegExpConcatenation ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" );

	UnboundedRegExpConcatenation * concatenation = new UnboundedRegExpConcatenation ( );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		concatenation->appendElement ( parseUnboundedRegExpElement ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "concatenation" );
	return std::rvalue_ref < UnboundedRegExpElement > ( concatenation );
}

std::rvalue_ref < UnboundedRegExpElement > RegExpFromXMLParser::parseUnboundedRegExpIteration ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	std::rvalue_ref < UnboundedRegExpElement > iteration ( new UnboundedRegExpIteration ( parseUnboundedRegExpElement ( input ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );

	return iteration;
}

std::rvalue_ref < UnboundedRegExpElement > RegExpFromXMLParser::parseUnboundedRegExpEpsilon ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" );

	std::rvalue_ref < UnboundedRegExpElement > epsilon ( new UnboundedRegExpEpsilon ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "epsilon" );

	return epsilon;
}

std::rvalue_ref < UnboundedRegExpElement > RegExpFromXMLParser::parseUnboundedRegExpEmpty ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	std::rvalue_ref < UnboundedRegExpElement > empty ( new UnboundedRegExpEmpty ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

std::rvalue_ref < FormalRegExpElement > RegExpFromXMLParser::parseFormalRegExpElement ( std::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseFormalRegExpEmpty ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" ) )
		return parseFormalRegExpEpsilon ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseFormalRegExpIteration ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseFormalRegExpAlternation ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" ) )
		return parseFormalRegExpConcatenation ( input );
	else
		return std::rvalue_ref < FormalRegExpElement > ( new FormalRegExpSymbol ( alib::xmlApi < alphabet::Symbol >::parse ( input ) ) );
}

std::rvalue_ref < FormalRegExpElement > RegExpFromXMLParser::parseFormalRegExpAlternation ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	std::rvalue_ref < FormalRegExpElement > element1 = parseFormalRegExpElement ( input );
	std::rvalue_ref < FormalRegExpElement > element2 = parseFormalRegExpElement ( input );

	std::rvalue_ref < FormalRegExpElement > alternation ( new FormalRegExpAlternation ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );

	return alternation;
}

std::rvalue_ref < FormalRegExpElement > RegExpFromXMLParser::parseFormalRegExpConcatenation ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" );

	std::rvalue_ref < FormalRegExpElement > element1 = parseFormalRegExpElement ( input );
	std::rvalue_ref < FormalRegExpElement > element2 = parseFormalRegExpElement ( input );

	std::rvalue_ref < FormalRegExpElement > concatenation ( new FormalRegExpConcatenation ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "concatenation" );

	return concatenation;
}

std::rvalue_ref < FormalRegExpElement > RegExpFromXMLParser::parseFormalRegExpIteration ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	std::rvalue_ref < FormalRegExpElement > iteration ( new FormalRegExpIteration ( parseFormalRegExpElement ( input ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );

	return iteration;
}

std::rvalue_ref < FormalRegExpElement > RegExpFromXMLParser::parseFormalRegExpEpsilon ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" );

	std::rvalue_ref < FormalRegExpElement > epsilon ( new FormalRegExpEpsilon ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "epsilon" );

	return epsilon;
}

std::rvalue_ref < FormalRegExpElement > RegExpFromXMLParser::parseFormalRegExpEmpty ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	std::rvalue_ref < FormalRegExpElement > empty ( new FormalRegExpEmpty ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

} /* namespace regexp */
