/*
 * RegExpFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_FROM_XML_PARSER_H_
#define REG_EXP_FROM_XML_PARSER_H_

#include <set>
#include <deque>
#include <memory>
#include "../RegExpFeatures.h"
#include "../../alphabet/SymbolFeatures.h"
#include <sax/Token.h>

namespace regexp {

/**
 * Parser used to get RegExp from XML parsed into list of tokens.
 */
class RegExpFromXMLParser {
public:
	static std::set < alphabet::Symbol > parseAlphabet ( std::deque < sax::Token >::iterator & input );

	static std::rvalue_ref < UnboundedRegExpElement > parseUnboundedRegExpElement ( std::deque < sax::Token >::iterator & input );

	static std::rvalue_ref < UnboundedRegExpElement > parseUnboundedRegExpEpsilon ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < UnboundedRegExpElement > parseUnboundedRegExpEmpty ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < UnboundedRegExpElement > parseUnboundedRegExpIteration ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < UnboundedRegExpElement > parseUnboundedRegExpAlternation ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < UnboundedRegExpElement > parseUnboundedRegExpConcatenation ( std::deque < sax::Token >::iterator & input );

	static std::rvalue_ref < FormalRegExpElement > parseFormalRegExpElement ( std::deque < sax::Token >::iterator & input );

	static std::rvalue_ref < FormalRegExpElement > parseFormalRegExpEpsilon ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < FormalRegExpElement > parseFormalRegExpEmpty ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < FormalRegExpElement > parseFormalRegExpIteration ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < FormalRegExpElement > parseFormalRegExpAlternation ( std::deque < sax::Token >::iterator & input );
	static std::rvalue_ref < FormalRegExpElement > parseFormalRegExpConcatenation ( std::deque < sax::Token >::iterator & input );
};

} /* namespace regexp */

#endif /* REG_EXP_FROM_XML_PARSER_H_ */
