/*
 * FormalRegExpAlternation.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExpAlternation.h"
#include <exception/CommonException.h>
#include "../unbounded/UnboundedRegExpAlternation.h"
#include <sstream>

namespace regexp {

FormalRegExpAlternation::FormalRegExpAlternation ( FormalRegExpElement && leftElement, FormalRegExpElement && rightElement ) : BinaryNode < std::smart_ptr < FormalRegExpElement >, std::smart_ptr < const FormalRegExpElement >, FormalRegExpAlternation > ( std::smart_ptr < FormalRegExpElement > ( std::move ( leftElement ).plunder ( ) ), std::smart_ptr < FormalRegExpElement > ( std::move ( rightElement ).plunder ( ) ) ) {
}

FormalRegExpAlternation::FormalRegExpAlternation ( const FormalRegExpElement & left, const FormalRegExpElement & right ) : FormalRegExpAlternation ( std::move_copy ( left ), std::move_copy ( right ) ) {
}

const FormalRegExpElement & FormalRegExpAlternation::getLeftElement ( ) const {
	return * getLeft ( );
}

const FormalRegExpElement & FormalRegExpAlternation::getRightElement ( ) const {
	return * getRight ( );
}

FormalRegExpElement & FormalRegExpAlternation::getLeftElement ( ) {
	return * getLeft ( );
}

FormalRegExpElement & FormalRegExpAlternation::getRightElement ( ) {
	return * getRight ( );
}

void FormalRegExpAlternation::setLeftElement ( FormalRegExpElement && element ) {
	setLeft ( std::smart_ptr < FormalRegExpElement > ( std::move ( element ).plunder ( ) ) );
}

void FormalRegExpAlternation::setLeftElement ( const FormalRegExpElement & element ) {
	setLeftElement ( std::move_copy ( element ) );
}

void FormalRegExpAlternation::setRightElement ( FormalRegExpElement && element ) {
	setRight ( std::smart_ptr < FormalRegExpElement > ( std::move ( element ).plunder ( ) ) );
}

void FormalRegExpAlternation::setRightElement ( const FormalRegExpElement & element ) {
	setRightElement ( std::move_copy ( element ) );
}

FormalRegExpElement * FormalRegExpAlternation::clone ( ) const {
	return new FormalRegExpAlternation ( * this );
}

FormalRegExpElement * FormalRegExpAlternation::plunder ( ) && {
	return new FormalRegExpAlternation ( std::move ( * this ) );
}

std::smart_ptr < UnboundedRegExpElement > FormalRegExpAlternation::asUnbounded ( ) const {
	UnboundedRegExpAlternation * res = new UnboundedRegExpAlternation ( );

	res->appendElement ( std::move ( * getLeftElement ( ).asUnbounded ( ) ) );
	res->appendElement ( std::move ( * getRightElement ( ).asUnbounded ( ) ) );

	return std::smart_ptr < UnboundedRegExpElement > ( res );
}

int FormalRegExpAlternation::compare ( const FormalRegExpAlternation & other ) const {
	int res = getLeftElement ( ).compare ( other.getLeftElement ( ) );

	if ( res == 0 ) res = getRightElement ( ).compare ( other.getRightElement ( ) );

	return res;
}

void FormalRegExpAlternation::operator >>( std::ostream & out ) const {
	out << "(FormalRegExpAlternation";
	out << " " << getLeft ( );
	out << " " << getRight ( );
	out << ")";
}

bool FormalRegExpAlternation::testSymbol ( const alphabet::Symbol & symbol ) const {
	if ( getLeft ( )->testSymbol ( symbol ) ) return true;

	if ( getRight ( )->testSymbol ( symbol ) ) return true;

	return false;
}

void FormalRegExpAlternation::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	getLeftElement ( ).computeMinimalAlphabet ( alphabet );
	getRightElement ( ).computeMinimalAlphabet ( alphabet );
}

bool FormalRegExpAlternation::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	return getLeftElement ( ).checkAlphabet ( alphabet ) && getRightElement ( ).checkAlphabet ( alphabet );
}

FormalRegExpAlternation::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
