/*
 * FormalRegExpConcatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#ifndef FORMAL_REG_EXP_CONCATENATION_H_
#define FORMAL_REG_EXP_CONCATENATION_H_

#include <vector>
#include <sstream>

#include "FormalRegExpElement.h"

namespace regexp {

/**
 * Represents concatenation operator in the regular expression. Contains list of FormalRegExpElement
 * as operands of the operator.
 */
class FormalRegExpConcatenation : public FormalRegExpElement, public std::BinaryNode < std::smart_ptr < FormalRegExpElement >, std::smart_ptr < const FormalRegExpElement >, FormalRegExpConcatenation > {
public:
	void Accept ( void * userData, const FormalRegExpElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRegExpConcatenation ( FormalRegExpElement && left, FormalRegExpElement && right );
	explicit FormalRegExpConcatenation ( const FormalRegExpElement & left, const FormalRegExpElement & right );

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement * clone ( ) const;

	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement * plunder ( ) &&;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual std::smart_ptr < UnboundedRegExpElement > asUnbounded ( ) const;

	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @copydoc FormalRegExpElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @return elements
	 */
	const FormalRegExpElement & getLeftElement ( ) const;
	const FormalRegExpElement & getRightElement ( ) const;

	/**
	 * @return elements
	 */
	FormalRegExpElement & getLeftElement ( );
	FormalRegExpElement & getRightElement ( );

	/**
	 * @param element to append
	 */
	void setLeftElement ( FormalRegExpElement && element );
	void setLeftElement ( const FormalRegExpElement & element );

	void setRightElement ( FormalRegExpElement && element );
	void setRightElement ( const FormalRegExpElement & element );

	virtual int compare ( const FormalRegExpElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRegExpConcatenation & ) const;

	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_CONCATENATION_H_ */
