/*
 * FormalRegExpIteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "FormalRegExpIteration.h"
#include <exception/CommonException.h>

#include "../unbounded/UnboundedRegExpIteration.h"
#include <sstream>

namespace regexp {

FormalRegExpIteration::FormalRegExpIteration ( FormalRegExpElement && element ) : UnaryNode < std::smart_ptr < FormalRegExpElement >, std::smart_ptr < const FormalRegExpElement >, FormalRegExpIteration > ( std::smart_ptr < FormalRegExpElement > ( std::move ( element ).plunder ( ) ) ) {
}

FormalRegExpIteration::FormalRegExpIteration ( const FormalRegExpElement & element ) : FormalRegExpIteration ( std::move_copy ( element ) ) {
}

const FormalRegExpElement & FormalRegExpIteration::getElement ( ) const {
	return * getChild ( );
}

FormalRegExpElement & FormalRegExpIteration::getElement ( ) {
	return * getChild ( );
}

void FormalRegExpIteration::setElement ( FormalRegExpElement && elementParam ) {
	setChild ( std::smart_ptr < FormalRegExpElement > ( std::move ( elementParam ).plunder ( ) ) );
}

void FormalRegExpIteration::setElement ( const FormalRegExpElement & elementParam ) {
	setElement ( std::move_copy ( elementParam ) );
}

FormalRegExpElement * FormalRegExpIteration::clone ( ) const {
	return new FormalRegExpIteration ( * this );
}

FormalRegExpElement * FormalRegExpIteration::plunder ( ) && {
	return new FormalRegExpIteration ( std::move ( * this ) );
}

std::smart_ptr < UnboundedRegExpElement > FormalRegExpIteration::asUnbounded ( ) const {
	return std::smart_ptr < UnboundedRegExpElement > ( new UnboundedRegExpIteration ( * getElement ( ).asUnbounded ( ) ) );
}

int FormalRegExpIteration::compare ( const FormalRegExpIteration & other ) const {
	return getElement ( ).compare ( other.getElement ( ) );
}

void FormalRegExpIteration::operator >>( std::ostream & out ) const {
	out << "(RegExpFormalRegExpIteration " << getElement ( ) << ")";
}

bool FormalRegExpIteration::testSymbol ( const alphabet::Symbol & symbol ) const {
	return getChild ( )->testSymbol ( symbol );
}

void FormalRegExpIteration::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	getElement ( ).computeMinimalAlphabet ( alphabet );
}

bool FormalRegExpIteration::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	return getElement ( ).checkAlphabet ( alphabet );
}

FormalRegExpIteration::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
