/*
 * FormalRegExpIteration.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef FORMAL_REG_EXP_ITERATION_H_
#define FORMAL_REG_EXP_ITERATION_H_

#include "FormalRegExpElement.h"
#include "FormalRegExpEmpty.h"

namespace regexp {

/**
 * Represents iteration operator in the regular expression. Contains one FormalRegExpElement
 * as operand.
 */
class FormalRegExpIteration : public FormalRegExpElement, public std::UnaryNode < std::smart_ptr < FormalRegExpElement >, std::smart_ptr < const FormalRegExpElement >, FormalRegExpIteration > {
public:
	void Accept ( void * userData, const FormalRegExpElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRegExpIteration ( FormalRegExpElement && );
	explicit FormalRegExpIteration ( const FormalRegExpElement & );

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement * clone ( ) const;

	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement * plunder ( ) &&;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual std::smart_ptr < UnboundedRegExpElement > asUnbounded ( ) const;

	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @copydoc FormalRegExpElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @return element
	 */
	const FormalRegExpElement & getElement ( ) const;

	/**
	 * @return element
	 */
	FormalRegExpElement & getElement ( );

	/**
	 * @param element to iterate
	 */
	void setElement ( FormalRegExpElement && element );
	void setElement ( const FormalRegExpElement & element );

	virtual int compare ( const FormalRegExpElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRegExpIteration & ) const;

	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_ITERATION_H_ */
