/*
 * UnboundedRegExpAlternation.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpAlternation.h"
#include <exception/CommonException.h>
#include "../formal/FormalRegExpAlternation.h"
#include "../formal/FormalRegExpEmpty.h"
#include <sstream>

namespace regexp {

UnboundedRegExpAlternation::UnboundedRegExpAlternation ( ) {
}

const std::vector < std::smart_ptr < const UnboundedRegExpElement > > & UnboundedRegExpAlternation::getElements ( ) const {
	return getChildren();
}

const std::vector < std::smart_ptr < UnboundedRegExpElement > > & UnboundedRegExpAlternation::getElements ( ) {
	return getChildren();
}

void UnboundedRegExpAlternation::appendElement ( UnboundedRegExpElement && element ) {
	pushBackChild ( std::smart_ptr < UnboundedRegExpElement > ( std::move ( element ).plunder ( ) ) );
}

void UnboundedRegExpAlternation::appendElement ( const UnboundedRegExpElement & element ) {
	appendElement ( std::move_copy ( element ) );
}

UnboundedRegExpElement * UnboundedRegExpAlternation::clone ( ) const {
	return new UnboundedRegExpAlternation ( * this );
}

UnboundedRegExpElement * UnboundedRegExpAlternation::plunder ( ) && {
	return new UnboundedRegExpAlternation ( std::move ( * this ) );
}

std::smart_ptr < FormalRegExpElement > UnboundedRegExpAlternation::asFormal ( ) const {
	if ( getElements ( ).size ( ) == 0 ) return std::smart_ptr < FormalRegExpElement > ( new FormalRegExpEmpty ( ) );

	std::smart_ptr < FormalRegExpElement > res = getElements ( )[getElements ( ).size ( ) - 1]->asFormal ( );

	for ( unsigned i = getElements ( ).size ( ) - 1; i >= 1; i-- )
		res = std::smart_ptr < FormalRegExpElement > ( new FormalRegExpAlternation ( std::move ( * getElements ( )[i - 1]->asFormal ( ) ), std::move ( * res ) ) );

	return res;
}

int UnboundedRegExpAlternation::compare ( const UnboundedRegExpAlternation & other ) const {
	std::compare < std::decay < decltype ( getElements ( ) ) >::type > comp;

	return comp ( getElements ( ), other.getElements ( ) );
}

void UnboundedRegExpAlternation::operator >>( std::ostream & out ) const {
	out << "(UnboundedRegExpAlternation";

	for ( const auto & child : getElements() )
		out << " " << * child;

	out << ")";
}

bool UnboundedRegExpAlternation::testSymbol ( const alphabet::Symbol & symbol ) const {
	for ( const auto & child : getElements() )
		if ( child->testSymbol ( symbol ) ) return true;

	return false;
}

bool UnboundedRegExpAlternation::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	for ( const auto & child : getElements() )
		if ( !child->checkAlphabet ( alphabet ) ) return false;

	return true;
}


void UnboundedRegExpAlternation::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	for ( const auto & child : getElements() )
		child->computeMinimalAlphabet ( alphabet );
}

UnboundedRegExpAlternation::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
