/*
 * UnboundedRegExpConcatenation.cpp
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpConcatenation.h"
#include <exception/CommonException.h>
#include "../formal/FormalRegExpConcatenation.h"
#include "../formal/FormalRegExpEpsilon.h"
#include <sstream>

namespace regexp {

UnboundedRegExpConcatenation::UnboundedRegExpConcatenation ( ) {
}

const std::vector < std::smart_ptr < const UnboundedRegExpElement > > & UnboundedRegExpConcatenation::getElements ( ) const {
	return getChildren();
}

const std::vector < std::smart_ptr < UnboundedRegExpElement > > & UnboundedRegExpConcatenation::getElements ( ) {
	return getChildren();
}

void UnboundedRegExpConcatenation::appendElement ( UnboundedRegExpElement && element ) {
	pushBackChild ( std::smart_ptr < UnboundedRegExpElement > ( std::move ( element ).plunder ( ) ) );
}

void UnboundedRegExpConcatenation::appendElement ( const UnboundedRegExpElement & element ) {
	appendElement ( std::move_copy ( element ) );
}

UnboundedRegExpElement * UnboundedRegExpConcatenation::clone ( ) const {
	return new UnboundedRegExpConcatenation ( * this );
}

UnboundedRegExpElement * UnboundedRegExpConcatenation::plunder ( ) && {
	return new UnboundedRegExpConcatenation ( std::move ( * this ) );
}

std::smart_ptr < FormalRegExpElement > UnboundedRegExpConcatenation::asFormal ( ) const {
	if ( getElements ( ).size ( ) == 0 ) return std::smart_ptr < FormalRegExpElement > ( new FormalRegExpEpsilon ( ) );

	std::smart_ptr < FormalRegExpElement > res = getElements ( )[getElements ( ).size ( ) - 1]->asFormal ( );

	for ( unsigned i = getElements ( ).size ( ) - 1; i >= 1; i-- )
		res = std::smart_ptr < FormalRegExpElement > ( new FormalRegExpConcatenation ( std::move ( * getElements ( )[i - 1]->asFormal ( ) ), std::move ( * res ) ) );

	return res;
}

int UnboundedRegExpConcatenation::compare ( const UnboundedRegExpConcatenation & other ) const {
	std::compare < std::decay < decltype ( getElements ( ) ) >::type > comp;

	return comp ( getElements ( ), other.getElements ( ) );
}

void UnboundedRegExpConcatenation::operator >>( std::ostream & out ) const {
	out << "(UnboundedRegExpConcatenation";

	for ( const auto & child : getElements() )
		out << " " << * child;

	out << ")";
}

bool UnboundedRegExpConcatenation::testSymbol ( const alphabet::Symbol & symbol ) const {
	for ( const auto & child : getElements() )
		if ( child->testSymbol ( symbol ) ) return true;

	return false;
}

bool UnboundedRegExpConcatenation::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	for ( const auto & child : getElements() )
		if ( !child->checkAlphabet ( alphabet ) ) return false;

	return true;
}

void UnboundedRegExpConcatenation::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	for ( const auto & child : getElements() )
		child->computeMinimalAlphabet ( alphabet );
}

UnboundedRegExpConcatenation::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
