/*
 * UnboundedRegExpEpsilon.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef UNBOUNDED_REG_EXP_EPSILON_H_
#define UNBOUNDED_REG_EXP_EPSILON_H_

#include "UnboundedRegExpElement.h"

namespace regexp {

/**
 * Represents epsilon in the regular expression.
 */
class UnboundedRegExpEpsilon : public UnboundedRegExpElement, public std::NularyNode < std::smart_ptr < UnboundedRegExpElement >, std::smart_ptr < const UnboundedRegExpElement >, UnboundedRegExpSymbol > {
public:
	void Accept ( void * userData, const UnboundedRegExpElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit UnboundedRegExpEpsilon ( );

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement * clone ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement * plunder ( ) &&;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual std::smart_ptr < FormalRegExpElement > asFormal ( ) const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

	/**
	 * @copydoc UnboundedRegExpElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const;

	virtual int compare ( const UnboundedRegExpElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UnboundedRegExpEpsilon & ) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_EPSILON_H_ */
