/*
 * UnboundedRegExpIteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpIteration.h"
#include <exception/CommonException.h>

#include "../formal/FormalRegExpIteration.h"
#include <sstream>

namespace regexp {

UnboundedRegExpIteration::UnboundedRegExpIteration ( UnboundedRegExpElement && element ) : UnaryNode < std::smart_ptr < UnboundedRegExpElement >, std::smart_ptr < const UnboundedRegExpElement >, UnboundedRegExpIteration > ( std::smart_ptr < UnboundedRegExpElement > ( std::move ( element ).plunder ( ) ) ) {
}

UnboundedRegExpIteration::UnboundedRegExpIteration ( const UnboundedRegExpElement & element ) : UnboundedRegExpIteration ( std::move_copy ( element ) ) {
}

const UnboundedRegExpElement & UnboundedRegExpIteration::getElement ( ) const {
	return * getChild ( );
}

UnboundedRegExpElement & UnboundedRegExpIteration::getElement ( ) {
	return * getChild ( );
}

void UnboundedRegExpIteration::setElement ( UnboundedRegExpElement && elementParam ) {
	setChild ( std::smart_ptr < UnboundedRegExpElement > ( std::move ( elementParam ).plunder ( ) ) );
}

void UnboundedRegExpIteration::setElement ( const UnboundedRegExpElement & elementParam ) {
	setElement ( std::move_copy ( elementParam ) );
}

UnboundedRegExpElement * UnboundedRegExpIteration::clone ( ) const {
	return new UnboundedRegExpIteration ( * this );
}

UnboundedRegExpElement * UnboundedRegExpIteration::plunder ( ) && {
	return new UnboundedRegExpIteration ( std::move ( * this ) );
}

std::smart_ptr < FormalRegExpElement > UnboundedRegExpIteration::asFormal ( ) const {
	return std::smart_ptr < FormalRegExpElement > ( new FormalRegExpIteration ( std::move ( * getElement ( ).asFormal ( ) ) ) );
}

int UnboundedRegExpIteration::compare ( const UnboundedRegExpIteration & other ) const {
	return getElement ( ).compare ( other.getElement ( ) );
}

void UnboundedRegExpIteration::operator >>( std::ostream & out ) const {
	out << "(RegExpUnboundedRegExpIteration " << getElement ( ) << ")";
}

bool UnboundedRegExpIteration::testSymbol ( const alphabet::Symbol & symbol ) const {
	return getElement ( ).testSymbol ( symbol );
}

bool UnboundedRegExpIteration::checkAlphabet ( const std::set < alphabet::Symbol > & alphabet ) const {
	return getElement ( ).checkAlphabet ( alphabet );
}

void UnboundedRegExpIteration::computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const {
	getElement ( ).computeMinimalAlphabet ( alphabet );
}

UnboundedRegExpIteration::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */
