#ifndef RTE_ALPHABET_H_
#define RTE_ALPHABET_H_

#include <set>
#include "../../alphabet/RankedSymbol.h"

namespace rte {

/**
 * Abstract base class for all strings. Contains common elements of strings.
 */
class RTEAlphabet {
protected:
	std::set < alphabet::RankedSymbol > alphabetF, alphabetK;

public:
	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 */
	bool addSymbolToAlphabet ( alphabet::RankedSymbol symbol );
	bool addConstantSymbolToAlphabet ( alphabet::RankedSymbol symbol );

	/**
	 * Adds input symbols to input alphabet.
	 * @param symbols Symbol to add
	 */
	void addSymbolsToAlphabet ( const std::set < alphabet::RankedSymbol > & symbols );
	void addConstantSymbolsToAlphabet ( const std::set < alphabet::RankedSymbol > & symbols );

	/**
	 * Sets input symbols of the tree.
	 * @param symbols Symbols to set
	 */
	void setAlphabet ( std::set < alphabet::RankedSymbol > symbols );
	void setConstantAlphabet ( std::set < alphabet::RankedSymbol > symbols );

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	virtual bool removeSymbolFromAlphabet ( const alphabet::RankedSymbol & symbol ) = 0;
	virtual bool removeConstantSymbolFromAlphabet ( const alphabet::RankedSymbol & symbol ) = 0;

	/**
	 * @return the input alphabet
	 */
	const std::set < alphabet::RankedSymbol > & getAlphabet ( ) const;
	const std::set < alphabet::RankedSymbol > & getConstantAlphabet ( ) const;
};

} /* namespace rte */

#endif /* RTE_ALPHABET_H_ */
