#include "RTEToXMLComposer.h"

#include "../RTE.h"
#include "../RTEClasses.h"

// #include "../unbounded/UnboundedRTEElements.h"
#include "../formal/FormalRTEElements.h"
#include <XmlApi.hpp>

namespace rte {

/*
 * void RTEToXMLComposer::Visit(void* userData, const UnboundedRTEAlternation& alternation) const {
 *  std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);
 *
 *  out.emplace_back("alternation", sax::Token::TokenType::START_ELEMENT);
 *  for (const auto& element : alternation.getElements()) {
 *      element->Accept(userData, *this);
 *  }
 *  out.emplace_back("alternation", sax::Token::TokenType::END_ELEMENT);
 * }
 *
 * void RTEToXMLComposer::Visit(void* userData, const UnboundedRTEConcatenation& concatenation) const {
 *  std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);
 *
 *  out.emplace_back("concatenation", sax::Token::TokenType::START_ELEMENT);
 *  for (auto element : concatenation.getElements()) {
 *      element->Accept(userData, *this);
 *  }
 *  out.emplace_back("concatenation", sax::Token::TokenType::END_ELEMENT);
 *
 * }
 *
 * void RTEToXMLComposer::Visit(void* userData, const UnboundedRTEIteration& iteration) const {
 *  std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);
 *
 *  out.emplace_back("iteration", sax::Token::TokenType::START_ELEMENT);
 *  iteration.getElement().Accept(userData, *this);
 *  out.emplace_back("iteration", sax::Token::TokenType::END_ELEMENT);
 * }
 *
 * void RTEToXMLComposer::Visit(void* userData, const UnboundedRTESymbol& symbol) const {
 *  std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);
 *
 *  alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
 * }
 *
 * void RTEToXMLComposer::Visit(void* userData, const UnboundedRTEEpsilon&) const {
 *  std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);
 *
 *  out.emplace_back("epsilon", sax::Token::TokenType::START_ELEMENT);
 *  out.emplace_back("epsilon", sax::Token::TokenType::END_ELEMENT);
 * }
 *
 * void RTEToXMLComposer::Visit(void* userData, const UnboundedRTEEmpty&) const {
 *  std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);
 *
 *  out.emplace_back("empty", sax::Token::TokenType::START_ELEMENT);
 *  out.emplace_back("empty", sax::Token::TokenType::END_ELEMENT);
 * }
 */
void RTEToXMLComposer::Visit ( void * userData, const FormalRTEAlternation & alternation ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "alternation", sax::Token::TokenType::START_ELEMENT );
	alternation.getLeftElement ( ).Accept ( userData, * this );
	alternation.getRightElement ( ).Accept ( userData, * this );
	out.emplace_back ( "alternation", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTESubstitution & substitution ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "substitution", sax::Token::TokenType::START_ELEMENT );

	alib::xmlApi < alphabet::RankedSymbol >::compose ( out, substitution.getSubstitutionSymbol ( ).getSymbol ( ) );
	substitution.getLeftElement ( ).Accept ( userData, * this );
	substitution.getRightElement ( ).Accept ( userData, * this );

	out.emplace_back ( "substitution", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTEIteration & iteration ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "iteration", sax::Token::TokenType::START_ELEMENT );

	alib::xmlApi < alphabet::RankedSymbol >::compose ( out, iteration.getSubstitutionSymbol ( ).getSymbol ( ) );
	iteration.getElement ( ).Accept ( userData, * this );

	out.emplace_back ( "iteration", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTESymbol & symbol ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( sax::Token ( "symbol", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::RankedSymbol >::compose ( out, symbol.getSymbol ( ) );

	for ( const std::smart_ptr < const FormalRTESymbol > & element : symbol.getElements ( ) )
		element->Accept ( userData, * this );

	out.emplace_back ( sax::Token ( "symbol", sax::Token::TokenType::END_ELEMENT ) );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTEEmpty & ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "empty", sax::Token::TokenType::START_ELEMENT );
	out.emplace_back ( "empty", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::composeAlphabet ( std::deque < sax::Token > & out, const std::set < alphabet::RankedSymbol > & alphabetF, const std::set < alphabet::RankedSymbol > & alphabetK ) {
	out.emplace_back ( "alphabet", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : alphabetF )
		alib::xmlApi < alphabet::RankedSymbol >::compose ( out, symbol );

	out.emplace_back ( "alphabet", sax::Token::TokenType::END_ELEMENT );

	out.emplace_back ( "constantAlphabet", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : alphabetK )
		alib::xmlApi < alphabet::RankedSymbol >::compose ( out, symbol );

	out.emplace_back ( "constantAlphabet", sax::Token::TokenType::END_ELEMENT );
}

RTEToXMLComposer RTEToXMLComposer::RTE_TO_XML_COMPOSER;

} /* namespace rte */
