#include "FormalRTEIteration.h"
#include <exception/CommonException.h>

// #include "../unbounded/UnboundedRTEIteration.h"
#include <sstream>

namespace rte {

FormalRTEIteration::FormalRTEIteration ( FormalRTEElement && element, FormalRTESymbol substitutionSymbol ) : element ( nullptr ), substitutionSymbol ( nullptr ) {
	setElement ( std::move ( element ) );
	setSubstitutionSymbol ( std::move ( substitutionSymbol ) );
}

FormalRTEIteration::FormalRTEIteration ( const FormalRTEElement & element, FormalRTESymbol substitutionSymbol ) : FormalRTEIteration ( std::move_copy ( element ), std::move ( substitutionSymbol ) ) {
}

FormalRTEIteration::FormalRTEIteration ( const FormalRTEIteration & other ) : element ( other.element ), substitutionSymbol ( other.substitutionSymbol ) {
}

FormalRTEIteration::FormalRTEIteration ( FormalRTEIteration && other ) noexcept : element ( std::move ( other.element ) ), substitutionSymbol ( std::move ( other.substitutionSymbol ) ) {
	element->attachRTE ( NULL );
	substitutionSymbol->attachRTE ( NULL );
}

FormalRTEIteration & FormalRTEIteration::operator =( const FormalRTEIteration & other ) {
	if ( this == & other )
		return * this;

	* this = FormalRTEIteration ( other );

	return * this;
}

FormalRTEIteration & FormalRTEIteration::operator =( FormalRTEIteration && other ) noexcept {
	std::swap ( this->element, other.element );
	std::swap ( this->substitutionSymbol, other.substitutionSymbol );
	std::swap ( this->parentRTE, other.parentRTE );

	this->attachRTE ( other.parentRTE );

	return * this;
}

FormalRTEIteration::~FormalRTEIteration ( ) noexcept {
}

const FormalRTEElement & FormalRTEIteration::getElement ( ) const {
	return * element;
}

FormalRTEElement & FormalRTEIteration::getElement ( ) {
	return * element;
}

const FormalRTESymbol & FormalRTEIteration::getSubstitutionSymbol ( ) const {
	return * substitutionSymbol;
}

FormalRTESymbol & FormalRTEIteration::getSubstitutionSymbol ( ) {
	return * substitutionSymbol;
}

void FormalRTEIteration::setElement ( const FormalRTEElement & element ) {
	setElement ( std::move_copy ( element ) );
}

void FormalRTEIteration::setElement ( FormalRTEElement && elementParam ) {
	element = std::smart_ptr < FormalRTEElement > ( std::move ( elementParam ).plunder ( ) );

	if ( this->parentRTE && !element->attachRTE ( this->parentRTE ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

void FormalRTEIteration::setSubstitutionSymbol ( FormalRTESymbol symbol ) {
	substitutionSymbol = std::smart_ptr < FormalRTESymbol > ( std::move ( symbol ).plunder ( ) );

	if ( this->parentRTE && !substitutionSymbol->attachRTE ( this->parentRTE ) )
		throw exception::CommonException ( "Input symbols not in the alphabet." );
}

FormalRTEIteration * FormalRTEIteration::clone ( ) const {
	return new FormalRTEIteration ( * this );
}

FormalRTEIteration * FormalRTEIteration::plunder ( ) && {
	return new FormalRTEIteration ( std::move ( * this ) );
}

/*
 * UnboundedRTEElement* FormalRTEIteration::cloneAsUnbounded() const {
 *  UnboundedRTEElement* tmp = this->element->cloneAsUnbounded();
 *  UnboundedRTEElement* res = new UnboundedRTEIteration(std::move(*tmp));
 *  delete tmp;
 *  return res;
 * }
 */
int FormalRTEIteration::compare ( const FormalRTEIteration & other ) const {
	auto first = std::tie ( getElement ( ), getSubstitutionSymbol ( ) );
	auto second = std::tie ( other.getElement ( ), other.getSubstitutionSymbol ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

void FormalRTEIteration::operator >>( std::ostream & out ) const {
	out << "(RTEFormalRTEIteration " << * substitutionSymbol << " " << * element << ")";
}

bool FormalRTEIteration::testSymbol ( const alphabet::RankedSymbol & symbol ) const {
	return element->testSymbol ( symbol );
}

bool FormalRTEIteration::attachRTE ( const FormalRTE * regexp ) {
	if ( this->parentRTE == regexp ) return true;

	this->parentRTE = regexp;

	if ( !this->element->attachRTE ( regexp ) ) return false;

	return this->parentRTE->getConstantAlphabet ( ).find ( this->substitutionSymbol->getSymbol ( ) ) != this->parentRTE->getConstantAlphabet ( ).end ( );
}

void FormalRTEIteration::computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const {
	alphabetK.insert ( substitutionSymbol->getSymbol ( ) );
	element->computeMinimalAlphabet ( alphabetF, alphabetK );
}

FormalRTEIteration::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
