#ifndef FORMAL_RTE_ITERATION_H_
#define FORMAL_RTE_ITERATION_H_

#include "FormalRTEElement.h"
#include "FormalRTESymbol.h"
#include "FormalRTE.h"

namespace rte {

namespace simplify {

class RTEOptimize;

} /* namespace simplify */

/**
 * Represents iteration operator in the regular expression. Contains one FormalRTEElement
 * as operand.
 */
class FormalRTEIteration : public FormalRTEElement {
protected:
	std::smart_ptr < FormalRTEElement > element;
	std::smart_ptr < FormalRTESymbol > substitutionSymbol;

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */

	// virtual UnboundedRTEElement * cloneAsUnbounded ( ) const;

	/**
	 * @copydoc FormalRTEElement::attachRTE()
	 */
	virtual bool attachRTE ( const FormalRTE * regexp );

public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTEIteration ( FormalRTEElement &&, FormalRTESymbol );
	explicit FormalRTEIteration ( const FormalRTEElement &, FormalRTESymbol );

	FormalRTEIteration ( const FormalRTEIteration & other );
	FormalRTEIteration ( FormalRTEIteration && other ) noexcept;
	FormalRTEIteration & operator =( const FormalRTEIteration & other );
	FormalRTEIteration & operator =( FormalRTEIteration && other ) noexcept;
	virtual ~FormalRTEIteration ( ) noexcept;

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEIteration * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEIteration * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::RankedSymbol & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const;

	/**
	 * @return element
	 */
	const FormalRTEElement & getElement ( ) const;
	const FormalRTESymbol & getSubstitutionSymbol ( ) const;

	/**
	 * @return element
	 */
	FormalRTEElement & getElement ( );
	FormalRTESymbol & getSubstitutionSymbol ( );

	/**
	 * @param element to iterate
	 */
	void setElement ( const FormalRTEElement & element );
	void setElement ( FormalRTEElement && element );

	void setSubstitutionSymbol ( FormalRTESymbol element );

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTEIteration & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend class simplify::RTEOptimize;

	virtual explicit operator std::string ( ) const;
};

} /* namespace rte */

#endif /* FORMAL_RTE_ITERATION_H_ */
