#ifndef FORMAL_RTE_SUBSTITUTION_H_
#define FORMAL_RTE_SUBSTITUTION_H_

#include "FormalRTEElement.h"
#include "FormalRTESymbol.h"
#include "FormalRTE.h"

namespace rte {

/**
 * Represents concatenation operator in the regular expression. Contains list of FormalRTEElement
 * as operands of the operator.
 */
class FormalRTESubstitution : public FormalRTEElement {
protected:
	/**
	 * @copydoc FormalRTEElement::clone() const
	 */

	// virtual UnboundedRTEElement * cloneAsUnbounded ( ) const;

	std::smart_ptr < FormalRTEElement > left;
	std::smart_ptr < FormalRTEElement > right;
	std::smart_ptr < FormalRTESymbol > substitutionSymbol; // substite this in left by right

	/**
	 * @copydoc FormalRTEElement::attachRTE()
	 */
	virtual bool attachRTE ( const FormalRTE * regexp );

public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTESubstitution ( FormalRTEElement && left, FormalRTEElement && right, FormalRTESymbol substitutionSymbol );
	explicit FormalRTESubstitution ( const FormalRTEElement & left, const FormalRTEElement & right, FormalRTESymbol substitutionSymbol );

	FormalRTESubstitution ( const FormalRTESubstitution & other );
	FormalRTESubstitution ( FormalRTESubstitution && other ) noexcept;
	FormalRTESubstitution & operator =( const FormalRTESubstitution & other );
	FormalRTESubstitution & operator =( FormalRTESubstitution && other ) noexcept;
	virtual ~FormalRTESubstitution ( ) noexcept;

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESubstitution * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESubstitution * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const alphabet::RankedSymbol & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < alphabet::RankedSymbol > & alphabetF, std::set < alphabet::RankedSymbol > & alphabetK ) const;

	/**
	 * @return elements
	 */
	const FormalRTEElement & getLeftElement ( ) const;
	const FormalRTEElement & getRightElement ( ) const;
	const FormalRTESymbol & getSubstitutionSymbol ( ) const;

	/**
	 * @return elements
	 */
	FormalRTEElement & getLeftElement ( );
	FormalRTEElement & getRightElement ( );
	FormalRTESymbol & getSubstitutionSymbol ( );

	/**
	 * @param element to append
	 */
	void setLeftElement ( const FormalRTEElement & element );
	void setLeftElement ( FormalRTEElement && element );

	void setRightElement ( const FormalRTEElement & element );
	void setRightElement ( FormalRTEElement && element );

	void setSubstitutionSymbol ( FormalRTESymbol element );

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTESubstitution & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;
	virtual explicit operator std::string ( ) const;
};

} /* namespace rte */

#endif /* FORMAL_RTE_SUBSTITUTION_H_ */
