/*
 * Epsilon.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef EPSILON_H_
#define EPSILON_H_

#include "StringBase.h"
#include <set>
#include <vector>

#include "../alphabet/Symbol.h"
#include <core/components.hpp>

namespace string {

class GeneralAlphabet;

/**
 * Represents epsilon. Regular expression is stored
 * as a tree of EpsilonElement.
 */
class Epsilon : public StringBase, public std::Components < Epsilon, alphabet::Symbol, std::tuple < GeneralAlphabet >, std::tuple < > > {
public:
	Epsilon ( );

	Epsilon ( std::set < alphabet::Symbol > alphabet );

	virtual StringBase * clone ( ) const;
	virtual StringBase * plunder ( ) &&;

	virtual const std::set < alphabet::Symbol > & getAlphabet ( ) const {
		return accessComponent < GeneralAlphabet > ( ).get ( );
	}

	void extendAlphabet ( const std::set < alphabet::Symbol > & symbols ) {
		accessComponent < GeneralAlphabet > ( ).add( symbols );
	}

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::vector < alphabet::Symbol > & getContent ( ) const;

	/**
	 * @return true
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Epsilon & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static Epsilon EPSILON;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Epsilon";

		return xmlTagName;
	}

	static Epsilon parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	static std::vector < alphabet::Symbol > content;
};

} /* namespace string */

#endif /* EPSILON_H_ */
