/*
 * PrefixRankedPattern.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef PREFIX_RANKED_PATTERN_H_
#define PREFIX_RANKED_PATTERN_H_

#include <set>
#include <vector>
#include <core/components.hpp>

#include "../../alphabet/RankedSymbol.h"
#include "../RankedTreeBase.h"
#include "RankedNode.h"

namespace tree {

class RankedPattern;
class PrefixRankedTree;
class GeneralAlphabet;
class SubtreeWildcard;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class PrefixRankedPattern : public RankedTreeBase, public std::Components < PrefixRankedPattern, alphabet::RankedSymbol, std::tuple < GeneralAlphabet >, std::tuple < SubtreeWildcard > > {
	std::vector < alphabet::RankedSymbol > m_Data;

	static std::vector < alphabet::RankedSymbol > toPrefixRanked ( const RankedNode & node, const alphabet::RankedSymbol & subtreeWildcard );
	static void toPrefixRanked ( const RankedNode & node, const alphabet::RankedSymbol & subtreeWildcard, std::vector < alphabet::RankedSymbol > & linearNotation ) ;

public:
	explicit PrefixRankedPattern ( alphabet::RankedSymbol subtreeWildcard, std::set < alphabet::RankedSymbol > alphabet, std::vector < alphabet::RankedSymbol > data );
	explicit PrefixRankedPattern ( alphabet::RankedSymbol subtreeWildcard, std::vector < alphabet::RankedSymbol > data );
	explicit PrefixRankedPattern ( const PrefixRankedTree & tree );
	explicit PrefixRankedPattern ( const RankedPattern & tree );

	virtual RankedTreeBase * clone ( ) const;
	virtual RankedTreeBase * plunder ( ) &&;

	/**
	 * @return List of symbols forming tree (const version).
	 */
	const std::vector < alphabet::RankedSymbol > & getContent ( ) const;

	const std::set < alphabet::RankedSymbol > & getAlphabet ( ) const {
		return accessComponent < GeneralAlphabet > ( ).get ( );
	}

	void extendAlphabet ( const std::set < alphabet::RankedSymbol > & symbols ) {
		accessComponent < GeneralAlphabet > ( ).add ( symbols );
	}

	const alphabet::RankedSymbol & getSubtreeWildcard ( ) const {
		return accessElement < SubtreeWildcard > ( ).get ( );
	}

	void setContent ( std::vector < alphabet::RankedSymbol > data );

	void arityChecksum ( const std::vector < alphabet::RankedSymbol > & data );

	/**
	 * @return true if tree is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const PrefixRankedPattern & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "PrefixRankedPattern";

		return xmlTagName;
	}

	static PrefixRankedPattern parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace tree */

#endif /* PREFIX_RANKED_PATTERN_H_ */
