/*
 * UsefullStates.cpp
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "UsefullStates.h"

#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <set>
#include <map>
#include <queue>

namespace automaton {

namespace properties {

namespace efficient {

std::set<automaton::State> UsefullStates::usefullStates(const Automaton& automaton) {
	return dispatch(automaton.getData());
}

template<class T>
std::set<automaton::State> UsefullStates::usefullStates( const T & fsm ) {
	std::map<automaton::State, std::set<automaton::State>> reversedTransitions;
	for(const auto& transition : fsm.getTransitions())
		for(const automaton::State& to : transition.second)
			reversedTransitions[to].insert(transition.first.first);

	std::deque<automaton::State> queue ( fsm.getFinalStates( ).begin(), fsm.getFinalStates().end() );
	std::set<automaton::State> visited = fsm.getFinalStates( );

	while( !queue.empty() ) {
		const std::set<automaton::State>& to = reversedTransitions[queue.front()];
		queue.pop_front();

		for(const automaton::State& process : to)
			if(visited.insert(process).second) {
				queue.push_back(std::move(const_cast<automaton::State&>(process)));
			}
	}

	return visited;
}

auto UsefullStatesEpsilonNFA = UsefullStates::RegistratorWrapper<std::set<automaton::State>, automaton::EpsilonNFA>(UsefullStates::usefullStates);
auto UsefullStatesNFA = UsefullStates::RegistratorWrapper<std::set<automaton::State>, automaton::NFA>(UsefullStates::usefullStates);
auto UsefullStatesCompactNFA = UsefullStates::RegistratorWrapper<std::set<automaton::State>, automaton::CompactNFA>(UsefullStates::usefullStates);
auto UsefullStatesExtendedNFA = UsefullStates::RegistratorWrapper<std::set<automaton::State>, automaton::ExtendedNFA>(UsefullStates::usefullStates);
auto UsefullStatesMultiInitialStateNFA = UsefullStates::RegistratorWrapper<std::set<automaton::State>, automaton::MultiInitialStateNFA>(UsefullStates::usefullStates);

template<>
std::set<automaton::State> UsefullStates::usefullStates( const automaton::DFA & fsm ) {
	std::map<automaton::State, std::set<automaton::State>> reversedTransitions;
	for(const auto& transition : fsm.getTransitions())
		reversedTransitions[transition.second].insert(transition.first.first);

	std::deque<automaton::State> queue ( fsm.getFinalStates( ).begin(), fsm.getFinalStates().end() );
	std::set<automaton::State> visited = fsm.getFinalStates( );

	while( !queue.empty() ) {
		const std::set<automaton::State>& to = reversedTransitions[queue.front()];
		queue.pop_front();

		for(const automaton::State& process : to)
			if(visited.insert(process).second) {
				queue.push_back(std::move(const_cast<automaton::State&>(process)));
			}
	}

	return visited;
}

auto UsefullStatesDFA = UsefullStates::RegistratorWrapper<std::set<automaton::State>, automaton::DFA>(UsefullStates::usefullStates);

} /* namespace efficient */

} /* namespace properties */

} /* namespace automaton */

