/*
 * EpsilonRemoverOutgoing.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Travnicek
 */

#include "EpsilonRemoverOutgoing.h"

#include "../../properties/efficient/AllEpsilonClosure.h"

namespace automaton {

namespace simplify {

namespace efficient {

automaton::DFA EpsilonRemoverOutgoing::remove(const automaton::DFA& origFSM)
{
	return origFSM;
}

auto EpsilonRemoverOutgoingDFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::DFA, automaton::DFA>(EpsilonRemoverOutgoing::remove);

automaton::MultiInitialStateNFA EpsilonRemoverOutgoing::remove(const automaton::MultiInitialStateNFA& origFSM)
{
	return origFSM;
}

auto EpsilonRemoverOutgoingMultiInitialStateNFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::MultiInitialStateNFA>(EpsilonRemoverOutgoing::remove);

automaton::NFA EpsilonRemoverOutgoing::remove(const automaton::NFA& origFSM)
{
	return origFSM;
}

auto EpsilonRemoverOutgoingNFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::NFA, automaton::NFA>(EpsilonRemoverOutgoing::remove);

automaton::MultiInitialStateNFA EpsilonRemoverOutgoing::remove( const automaton::EpsilonNFA & origFSM ) {
	automaton::MultiInitialStateNFA fsm;
	fsm.setStates( origFSM.getStates() );
	fsm.setFinalStates( origFSM.getFinalStates() );
	fsm.setInputAlphabet( origFSM.getInputAlphabet() );

	std::map<automaton::State, std::set<automaton::State>> closures = automaton::properties::efficient::AllEpsilonClosure::allEpsilonClosure( origFSM );

	std::map<std::pair<automaton::State, alphabet::Symbol>, std::set<automaton::State>> transitions = origFSM.getSymbolTransitions();

	for(const std::pair<const std::pair<automaton::State, alphabet::Symbol>, std::set<automaton::State>>& transition : transitions)
		for(const automaton::State& to : transition.second)
			for(const automaton::State& toClosure : closures[to])
				fsm.addTransition(transition.first.first, transition.first.second, toClosure);

	/**
	 * Step 2 from Melichar 2.41
	 */
	const std::set<automaton::State>& cl = closures [ origFSM.getInitialState() ];
	fsm.setInitialStates( cl );

	return fsm;
}

auto EpsilonRemoverOutgoingEpsilonNFA = EpsilonRemoverOutgoing::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::EpsilonNFA>(EpsilonRemoverOutgoing::remove);

automaton::Automaton EpsilonRemoverOutgoing::remove(const automaton::Automaton& automaton) {
	return dispatch(automaton.getData());
}

} /*namespace efficient */

} /* namespace simplify */

} /* namespace automaton */
