/*
 * map.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __MAP_HPP_
#define __MAP_HPP_

namespace std {

template< class T, class R, class ... Ts >
std::ostream& operator<<(std::ostream& out, const std::map<T, R, Ts ... >& map) {
	out << "{";

	bool first = true;
	for(const std::pair<const T, R>& item : map) {
		if(!first) out << ", ";
		first = false;
		out << item;
	}

	out << "}";
	return out;
}

template<class T, class R, class ... Ts>
struct compare<map<T, R, Ts ...>> {
	int operator()(const map<T, R, Ts ...>& first, const map<T, R, Ts ...>& second) const {
		if(first.size() < second.size()) return -1;
		if(first.size() > second.size()) return 1;

		compare<std::pair<typename std::decay < T >::type, typename std::decay < R >::type >> comp;
		for(auto iterF = first.begin(), iterS = second.begin(); iterF != first.end(); ++iterF, ++iterS) {
			int res = comp(*iterF, *iterS);
			if(res != 0) return res;
		}
		return 0;
	}
};

template<typename map_type>
class key_iterator : public map_type::const_iterator {
public:
	typedef typename map_type::const_iterator map_iterator;
	typedef typename map_iterator::value_type::first_type key_type;

	key_iterator(const map_iterator& other) : map_iterator(other) {} ;

	const key_type& operator *() const {
		return map_iterator::operator*().first;
	}

	const key_type* operator ->() const {
		return &(map_iterator::operator*().first);
	}
};

// helpers to create iterators easier:
template<typename map_type>
key_iterator<map_type> key_begin(const map_type& m) {
	return key_iterator<map_type>(m.begin());
}

template<typename map_type>
key_iterator<map_type> key_end(const map_type& m) {
	return key_iterator<map_type>(m.end());
}

} /* namespace std */

#endif /* __MAP_HPP_ */

