/*
 * SymbolToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolToStringComposer.h"
#include "alphabet/LabeledSymbol.h"
#include "alphabet/StartSymbol.h"
#include "alphabet/EndSymbol.h"
#include "alphabet/BottomOfTheStackSymbol.h"
#include "alphabet/BlankSymbol.h"
#include "alphabet/BarSymbol.h"
#include "alphabet/VariablesBarSymbol.h"
#include "alphabet/SubtreeWildcardSymbol.h"
#include "alphabet/NonlinearVariableSymbol.h"
#include "alphabet/RankedSymbol.h"
#include "alphabet/SymbolPairSymbol.h"
#include "alphabet/SymbolSetSymbol.h"
#include "alphabet/UniqueSymbol.h"

#include "../StringApi.hpp"

namespace alphabet {

void SymbolToStringComposer::compose(std::ostream& out, const LabeledSymbol& symbol) {
	alib::stringApi<label::Label>::compose(out, symbol.getLabel());
}

SymbolToStringComposer::RegistratorWrapper<void, LabeledSymbol> SymbolToStringComposerLabeledSymbol = SymbolToStringComposer::RegistratorWrapper<void, LabeledSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const BlankSymbol&) {
	out << "#B";
}

SymbolToStringComposer::RegistratorWrapper<void, BlankSymbol> SymbolToStringComposerBlankSymbol = SymbolToStringComposer::RegistratorWrapper<void, BlankSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const BottomOfTheStackSymbol&) {
	out << "#T";
}

SymbolToStringComposer::RegistratorWrapper<void, BottomOfTheStackSymbol> SymbolToStringComposerBottomOfTheStackSymbol = SymbolToStringComposer::RegistratorWrapper<void, BottomOfTheStackSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const EndSymbol&) {
	out << "#$";
}

SymbolToStringComposer::RegistratorWrapper<void, EndSymbol> SymbolToStringComposerEndSymbol = SymbolToStringComposer::RegistratorWrapper<void, EndSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const StartSymbol&) {
	out << "#^";
}

SymbolToStringComposer::RegistratorWrapper<void, StartSymbol> SymbolToStringComposerStartSymbol = SymbolToStringComposer::RegistratorWrapper<void, StartSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const RankedSymbol& symbol) {
	alib::stringApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(symbol.getRank()));
}

SymbolToStringComposer::RegistratorWrapper<void, RankedSymbol> SymbolToStringComposerRankedSymbol = SymbolToStringComposer::RegistratorWrapper<void, RankedSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const BarSymbol&) {
	out << "#|";
}

SymbolToStringComposer::RegistratorWrapper<void, BarSymbol> SymbolToStringComposerBarSymbol = SymbolToStringComposer::RegistratorWrapper<void, BarSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const VariablesBarSymbol&) {
	out << "#/";
}

SymbolToStringComposer::RegistratorWrapper<void, VariablesBarSymbol> SymbolToStringComposerVariablesBarSymbol = SymbolToStringComposer::RegistratorWrapper<void, VariablesBarSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const SubtreeWildcardSymbol&) {
	out << "#S";
}

SymbolToStringComposer::RegistratorWrapper<void, SubtreeWildcardSymbol> SymbolToStringComposerSubtreeWildcardSymbol = SymbolToStringComposer::RegistratorWrapper<void, SubtreeWildcardSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const NonlinearVariableSymbol& symbol ) {
	out << "$";
	alib::stringApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
}

SymbolToStringComposer::RegistratorWrapper<void, NonlinearVariableSymbol> SymbolToStringComposerNonlinearVariableSymbol = SymbolToStringComposer::RegistratorWrapper<void, NonlinearVariableSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const SymbolSetSymbol& symbol) {
	out << '[';
	bool first = true;
	for(const Symbol& innerSymbol : symbol.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		dispatch(out, innerSymbol.getData());
	}
	out << ']';
}

SymbolToStringComposer::RegistratorWrapper<void, SymbolSetSymbol> SymbolToStringComposerSymbolSetSymbol = SymbolToStringComposer::RegistratorWrapper<void, SymbolSetSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const SymbolPairSymbol& symbol) {
	out << '<';
	dispatch(out, symbol.getData().first.getData());
	out << ", ";
	dispatch(out, symbol.getData().second.getData());
	out << '>';
}

SymbolToStringComposer::RegistratorWrapper<void, SymbolPairSymbol> SymbolToStringComposerSymbolPairSymbol = SymbolToStringComposer::RegistratorWrapper<void, SymbolPairSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const UniqueSymbol& symbol) {
	out << '<';
	dispatch(out, symbol.getSymbol().getData());
	out << ", ";
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(symbol.getId()));
	out << '>';
}

SymbolToStringComposer::RegistratorWrapper<void, UniqueSymbol> SymbolToStringComposerUniqueSymbol = SymbolToStringComposer::RegistratorWrapper<void, UniqueSymbol>(SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const Symbol& symbol) {
	dispatch(out, symbol.getData());
}

} /* namespace alphabet */
