/*
 * alphabetManip.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>
#include <tree/RankedTreeWrapper.h>
#include <tree/UnrankedTreeWrapper.h>
#include <string/String.h>
#include <container/ObjectsSet.h>

#include <iostream>

template < class T >
void process ( T & data, TCLAP::ValueArg < std::string > & operation, std::deque < sax::Token > && argumentTokens ) {
	typedef typename std::remove_const < typename std::remove_reference < decltype ( data.getAlphabet ( ) ) >::type >::type alphabetType;

	if ( operation.getValue ( ) == "get" ) {
		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		const alphabetType & res = data.getAlphabet ( );

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		alib::XmlDataFactory::toStdout ( res );
	} else if ( operation.getValue ( ) == "add" ) {
		alphabetType symbols = alib::XmlDataFactory::fromTokens < alphabetType > ( std::move ( argumentTokens ) );

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		data.extendAlphabet ( symbols );

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		alib::XmlDataFactory::toStdout ( data );
	} else if ( operation.getValue ( ) == "set" ) {
	} else if ( operation.getValue ( ) == "remove" ) {
	} else {
		throw exception::CommonException ( "Invalid operation" );
	}
}

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Alphabet manipulation binary", ' ', "0.01" );

		std::vector < std::string > allowed;
		allowed.push_back ( "get" );
		allowed.push_back ( "set" );
		allowed.push_back ( "add" );
		allowed.push_back ( "remove" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > operation ( "o", "operation", "Operation type", false, "", & allowedVals );
		cmd.add ( operation );

		TCLAP::ValueArg < std::string > input ( "i", "input", "Input from file", false, "-", "file" );
		cmd.add ( input );

		TCLAP::ValueArg < std::string > argument ( "a", "argument", "Operation argument", false, "", "file" );
		cmd.add ( argument );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::deque < sax::Token > inputTokens = sax::FromXMLParserHelper::parseInput(input);

		std::deque < sax::Token > argumentTokens;
		if ( argument.isSet ( ) )
			argumentTokens = sax::FromXMLParserHelper::parseInput( argument );

		if ( alib::XmlDataFactory::first < tree::RankedTreeWrapper > ( inputTokens ) ) {
			tree::RankedTreeWrapper data = alib::XmlDataFactory::fromTokens < tree::RankedTreeWrapper > ( std::move ( inputTokens ) );
			process ( data, operation, std::move ( argumentTokens ) );
		} else if ( alib::XmlDataFactory::first < tree::UnrankedTreeWrapper > ( inputTokens ) ) {
			tree::UnrankedTreeWrapper data = alib::XmlDataFactory::fromTokens < tree::UnrankedTreeWrapper > ( std::move ( inputTokens ) );
			process ( data, operation, std::move ( argumentTokens ) );
		} else if ( alib::XmlDataFactory::first < string::String > ( inputTokens ) ) {
			string::String data = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( inputTokens ) );
			process ( data, operation, std::move ( argumentTokens ) );
		} else {
			throw exception::CommonException ( "Invalid data type" );
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) std::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
