/*
 * AutomataStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "AutomataStat.h"
#include "common/Printer.hpp"

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>

#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/NPDA.h>

void AutomataStat::stat ( const automaton::Automaton & automaton, const AutomataSettings & settings ) {
	dispatch ( automaton.getData ( ), settings );
}

void AutomataStat::stat ( const automaton::NFA & automaton, const AutomataSettings & settings ) {
	Printer::PrintSet ( settings.states, automaton.getStates ( ) );
	Printer::PrintSet ( settings.finalStates, automaton.getFinalStates ( ) );
	Printer::PrintValue ( settings.initialStates, automaton.getInitialState ( ) );
	Printer::PrintSet ( settings.inputAlphabet, automaton.getInputAlphabet ( ) );
	Printer::PrintMap ( settings.transitions, automaton.getTransitions ( ) );
}

auto AutomataStatNFA = AutomataStat::RegistratorWrapper < void, automaton::NFA > ( AutomataStat::stat );

void AutomataStat::stat ( const automaton::DFA & automaton, const AutomataSettings & settings ) {
	Printer::PrintSet ( settings.states, automaton.getStates ( ) );
	Printer::PrintSet ( settings.finalStates, automaton.getFinalStates ( ) );
	Printer::PrintValue ( settings.initialStates, automaton.getInitialState ( ) );
	Printer::PrintSet ( settings.inputAlphabet, automaton.getInputAlphabet ( ) );
	Printer::PrintMap ( settings.transitions, automaton.getTransitions ( ) );
}

auto AutomataStatDFA = AutomataStat::RegistratorWrapper < void, automaton::DFA > ( AutomataStat::stat );

void AutomataStat::stat ( const automaton::InputDrivenNPDA & automaton, const AutomataSettings & settings ) {
	Printer::PrintSet ( settings.states, automaton.getStates ( ) );
	Printer::PrintSet ( settings.finalStates, automaton.getFinalStates ( ) );
	Printer::PrintValue ( settings.initialStates, automaton.getInitialState ( ) );
	Printer::PrintSet ( settings.inputAlphabet, automaton.getInputAlphabet ( ) );
	Printer::PrintMap ( settings.transitions, automaton.getTransitions ( ) );
}

auto AutomataStatInputDrivenNPDA = AutomataStat::RegistratorWrapper < void, automaton::InputDrivenNPDA > ( AutomataStat::stat );

void AutomataStat::stat ( const automaton::InputDrivenDPDA & automaton, const AutomataSettings & settings ) {
	Printer::PrintSet ( settings.states, automaton.getStates ( ) );
	Printer::PrintSet ( settings.finalStates, automaton.getFinalStates ( ) );
	Printer::PrintValue ( settings.initialStates, automaton.getInitialState ( ) );
	Printer::PrintSet ( settings.inputAlphabet, automaton.getInputAlphabet ( ) );
	Printer::PrintMap ( settings.transitions, automaton.getTransitions ( ) );
}

auto AutomataStatInputDrivenDPDA = AutomataStat::RegistratorWrapper < void, automaton::InputDrivenDPDA > ( AutomataStat::stat );

void AutomataStat::stat ( const automaton::NPDA & automaton, const AutomataSettings & settings ) {
	Printer::PrintSet ( settings.states, automaton.getStates ( ) );
	Printer::PrintSet ( settings.finalStates, automaton.getFinalStates ( ) );
	Printer::PrintValue ( settings.initialStates, automaton.getInitialState ( ) );
	Printer::PrintSet ( settings.inputAlphabet, automaton.getInputAlphabet ( ) );
	Printer::PrintMap ( settings.transitions, automaton.getTransitions ( ) );
}

auto AutomataStatNPDA = AutomataStat::RegistratorWrapper < void, automaton::NPDA > ( AutomataStat::stat );

void AutomataStat::stat ( const automaton::DPDA & automaton, const AutomataSettings & settings ) {
	Printer::PrintSet ( settings.states, automaton.getStates ( ) );
	Printer::PrintSet ( settings.finalStates, automaton.getFinalStates ( ) );
	Printer::PrintValue ( settings.initialStates, automaton.getInitialState ( ) );
	Printer::PrintSet ( settings.inputAlphabet, automaton.getInputAlphabet ( ) );
	Printer::PrintMap ( settings.transitions, automaton.getTransitions ( ) );
}

auto AutomataStatDPDA = AutomataStat::RegistratorWrapper < void, automaton::DPDA > ( AutomataStat::stat );
